' Alpha texture demo

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global RedBall:BODY
Global GreenBall:BODY
Global RedBallMesh:MESH
Global RedBallMeshShadow:MESH
Global GreenBallMesh:MESH
Global GreenBallMeshShadow:MESH
Global RedBallTerrain:BODYBODY
Global GreenBallTerrain:BODYBODY
Global RedBallGreenBall:BODYBODY

Global rendermode:Int

Global FontWhite:FONT

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(1024,768,32,True,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(camera,_D3DXVECTOR3(0.0,1.0,-10.0))
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	   iShadowColorSet(_D3DXVECTOR3(0.9,0.9,0.7))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   ' create red ball body
	   RedBall=iBodySGCreate("default_res\\ball_.spg",0.13)
	   ' create red ball meshes And attach them To red ball body
	   RedBallMesh=iBodyMeshCreate("default_res\\ballred.x",RedBall)
	   RedBallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",RedBall,25.0)
	   ' change red ball alpha texture
	   iMeshAlphaTextureChange(RedBallMesh,"default_res\\alpha_a.jpg")
	
	   ' create green ball body
	   GreenBall=iBodySGCreate("default_res\\ball_.spg",0.13)
	   ' create green ball meshes And attach them To green ball body
	   GreenBallMesh=iBodyMeshCreate("default_res\\ballgreen.x",GreenBall)
	   GreenBallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",GreenBall,25.0)
	   ' change green ball alpha texture
	   iMeshAlphaTextureChange(GreenBallMesh,"default_res\\alpha_a.jpg")
	
	   ' NOTE: redball.x And greenball.x meshes have been modeled with a Double surface.
	   ' That is, both the outside And the inside of the balls are visible. This is
	   ' evident when we apply the transparent (alpha_a.jpg) texture To them.
	
	   ' set red ball body initial location
	   iBodyLocationSet(RedBall,_D3DXVECTOR3(0.0,5.0,0.0),True)
	   iBodyDampingSet(RedBall,0.0,0.999,0.0,0.995)
	   ' set green ball body initial location
	   iBodyLocationSet(GreenBall,_D3DXVECTOR3(4.0,1.2,-5.0),True)
	   iBodyDampingSet(GreenBall,0.0,0.999,0.0,0.995)
	
	   ' enable collisions
	   RedBallTerrain=iBodyBodyCreate(RedBall,Terrain)
	   iBodyBodyFrictionSet(RedBallTerrain,10.0)
	   GreenBallTerrain=iBodyBodyCreate(GreenBall,Terrain)
	   iBodyBodyFrictionSet(GreenBallTerrain,10.0)
	   RedBallGreenBall=iBodyBodyCreate(RedBall,GreenBall)
	   iBodyBodyFrictionSet(RedBallGreenBall,1.0)
	
	   ' set initial render mode To 0
	   Rendermode = 0
	
	   ' enable bodies
	   iBodyEnable(Terrain)
	   iBodyEnable(RedBall)
	   iBodyEnable(GreenBall)
	
	   ' hide message And show all meshes
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(RedBallMesh)
	   iMeshShow(RedBallMeshShadow)
	   iMeshShow(GreenBallMesh)
	   iMeshShow(GreenBallMeshShadow)

	EndMethod

	Method _Run()
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(RedBall,cameratarget)
	   iCameraLookAt(camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3=_D3DXVECTOR3()
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration.x=0
	   acceleration.y=0
	   acceleration.z=0
	
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(RedBall,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		iBodyAccelerationApply(RedBall,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else 
		iBodyAccelerationApply(RedBall,_D3DXVECTOR3(0.0,9.80665,0.0))
        EndIf
	
	   iBodyAccelerationApply(GreenBall,_D3DXVECTOR3(0.0,-9.80665,0.0))
	
	   ' Set the rendering priority For the two ball meshes depending on their
	   ' distance from the camera, so that the farther ball is rendered first.
	
	   ' define some temporary vectors
	   Local cameralocation:D3DXVECTOR3=_D3DXVECTOR3()
	   Local redlocation:D3DXVECTOR3=_D3DXVECTOR3()
	   Local greenlocation:D3DXVECTOR3=_D3DXVECTOR3()

	   ' And store current camera And ball locations in them
	   iCameraLocation(camera,cameralocation)
	   iMeshLocation(RedBallMesh,redlocation)
	   iMeshLocation(GreenBallMesh,greenlocation)
	
	   ' compare the [squared] lengths of vector from camera To red ball And
	   ' from camera To green ball. Note that &(redlocation-cameralocation)
	   ' is an expression that calculates the vector from one point To another
	   ' And resolves To a pointer To a temporary variable allocated somewhere
	   ' by the compiler. It is basically an advanced C++ trick... ;-)
	   If (iVectorLengthSq(redlocation.Minus(cameralocation)) > iVectorLengthSq(greenlocation.Minus(cameralocation)))
	      ' red ball is farther
	      iPrintCentered("Red ball is farther: rendered first",_D3DXVECTOR2(0.0,11.0),_D3DXVECTOR2(1.0,0.5),0.1,FontWhite,0.0)
	      iMeshRenderPrioritySet(RedBallMesh,1) ' render this mesh...
	      iMeshRenderPrioritySet(GreenBallMesh,2) ' ...before this mesh
	   Else
	      ' green ball is farther
	      iPrintCentered("Green ball is farther: rendered first",_D3DXVECTOR2(0.0,11.0),_D3DXVECTOR2(1.0,0.5),0.1,FontWhite,0.0)
	      iMeshRenderPrioritySet(RedBallMesh,2) ' render this mesh...
	      iMeshRenderPrioritySet(GreenBallMesh,1) ' ...after this mesh
	   EndIf
	
	   ' Select render mode For the balls according To current Rendermode variable state
	   If (Rendermode = 0)
	      ' render the balls in normal mode
	      iMeshRenderModeSet(RedBallMesh,D3DBLEND_SRCALPHA,D3DBLEND_INVSRCALPHA,True,False)
	      iMeshRenderModeSet(GreenBallMesh,D3DBLEND_SRCALPHA,D3DBLEND_INVSRCALPHA,True,False)
	   Else
	      ' render the balls in 'burnout' mode
	      iMeshRenderModeSet(RedBallMesh,D3DBLEND_SRCALPHA,D3DBLEND_ONE,False,False)
	      iMeshRenderModeSet(GreenBallMesh,D3DBLEND_SRCALPHA,D3DBLEND_ONE,False,False)
	   EndIf
	
	   ' switch render modes on [space] key press
	   If (iKeyClick(DIK_SPACE)) Then Rendermode = Not RenderMode ' If 0 turns 1, If 1 turns 0
	   iPrintCentered("PRESS [SPACE] TO SWITCH RENDERING MODE",_D3DXVECTOR2(0.0,10.0),_D3DXVECTOR2(0.4,0.35),0.1,FontWhite,0.0)
	EndMethod

EndType

