'  ALPHA TEXTURES UNLIMITED DEMO.
'  By Damian Rajkowski (WeirdPumpkin).

'  This demo is based on the AlphaTexture demo. It has been modified To handle any number of masked objects,
'  automatically calculating the correct rendering priority order with a fast linked-list algorithm.
'  If you can find a better way, either by improving the sorting Or finding a workaround which eliminates the
'  entire need For sorting, please let me know.

'  The user controls the red ball. Move the red ball around And between the other balls, And knock them around.
'  You should find that distant balls are always visible through closer balls, regardless of which balls obscure which.
'  If you find a Case where this isn't so - let me know as well.

'  You can increase the number of balls easily. Just set the Balls constant, And specify locations For the balls.
'  Apart from that, the rest is handled automatically.

'  THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict

Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH

' ************************************************
'  ADDED For THIS DEMO
' ************************************************
Const Balls:Int   = 5          '  Specify number of balls in scene.
Const Couples:Int = 10         '  5 objects - 10 unique connections. Count 'em.

Global Ball:BODY[Balls]
Global BallMesh:MESH[Balls]
Global ShadowMesh:MESH[Balls]
Global BallTerrain:BODYBODY[Balls]
Global BallBall:BODYBODY[Couples]
Global BallDist:Float[Balls]

'  Linked-list structure For quickly ordering objects For rendering.
Type _NODE 
    Field prev:_NODE     '  Reference To preceeding node, If any, Or NULL.
    Field _next:_NODE    '  Reference To Next node, If any, Or NULL.
    Field mesh:MESH     '  Reference To mesh at this node.
    Field dist:Float        '  Distance between camera And mesh.
EndType

Global Node:_NODE[Balls]

' init the nodes
For Local i:Int=0 To Balls-1
	Node[i]=New _NODE
Next

Global first:_NODE
Global Nodes:Int         '  Number of nodes used so far.

' ************************************************

Global rendermode:Int

Global FontWhite:FONT

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	    Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	    iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,1.0,-10.0))
	
	    FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	    iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	    iShadowColorSet(_D3DXVECTOR3(0.9,0.9,0.7))
	
	    LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	    iSpriteRender(LoadingMessage,False)
	
	    SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	    Terrain     = iBodyCreate    ("default_res\\terrain_5.00.ply")
	    TerrainMesh = iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	    ' ************************************************
	    '  ADDED For THIS DEMO
	    ' ************************************************
	    Local i:Int,j:Int,n:Int
	
	    '  Create balls.
	    For i = 0 To Balls-1
	        Ball[i] = iBodySGCreate("default_res\\ball_.spg",0.13)
	        If (i = 0)
	            '  First ball is red - And user-controlled.
	            BallMesh[i] = iBodyMeshCreate("default_res\\ballred.x",  Ball[i])
	        Else
	            '  Remaining balls are green - And Not user-controlled.
	            BallMesh[i] = iBodyMeshCreate("default_res\\ballgreen.x",Ball[i])
	        EndIf
	        ShadowMesh[i] = iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball[i],25.0)
	        iMeshAlphaTextureChange(BallMesh[i],"default_res\\alpha_a.jpg")
	        iBodyDampingSet(Ball[i],0.0,0.999,0.0,0.995)
	        BallTerrain[i] = iBodyBodyCreate(Ball[i],Terrain)
	        iBodyBodyFrictionSet(BallTerrain[i],10.0)
	    Next
	
	    '  Set ball locations.
	    iBodyLocationSet(Ball[0],_D3DXVECTOR3( 0.0,1.2, 0.0),True)  '  Red ball - user controlled.
	    iBodyLocationSet(Ball[1],_D3DXVECTOR3( 4.0,1.2,-5.0),True)
	    iBodyLocationSet(Ball[2],_D3DXVECTOR3(-4.0,1.2,-5.0),True)
	    iBodyLocationSet(Ball[3],_D3DXVECTOR3( 4.0,1.2, 5.0),True)
	    iBodyLocationSet(Ball[4],_D3DXVECTOR3(-4.0,1.2, 5.0),True)
	
	    '  Create collsion couples.
	    n = 0
	    For i = 0 To Balls-1
	        For j = 0 To Balls-1
	            If (j > i) 
	                '  Create only unique collision couples.
	                BallBall[n] = iBodyBodyCreate(Ball[i],Ball[j])
	                iBodyBodyFrictionSet(BallBall[n],1.0)
	                n:+1
	            EndIf
	        Next
	    Next
	
	    '  Set initial render mode To 0
	    Rendermode = 0
	
	    '  Hide message And show all meshes
	    iSpriteHide(LoadingMessage)
	    iSkyBoxShow(SkyBox)
	    iBodyEnable(Terrain)
	    iMeshShow(TerrainMesh)
	    ' ************************************************
	    '  MODIFIED For THIS DEMO.
	    ' ************************************************
	    For i = 0 To Balls-1
	        iBodyEnable(Ball      [i])
	        iMeshShow  (BallMesh  [i])
	        iMeshShow  (ShadowMesh[i])
	    Next
	    ' ************************************************
	EndMethod

	Method _Run()
	    Local i:Int
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball[0],cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3=_D3DXVECTOR3()
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration.x=0
	   acceleration.y=0
	   acceleration.z=0
	
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	    '  Apply user-controlled accelerations To red ball.
	    iBodyAccelerationApply(Ball[0],acceleration)
	
	    '  Apply gravity To red ball, as dictated by user.
	    If Not iKeyDown(DIK_LCONTROL)
	       iBodyAccelerationApply(Ball[0],_D3DXVECTOR3(0.0,-9.80665,0.0))
	    Else 
	       iBodyAccelerationApply(Ball[0],_D3DXVECTOR3(0.0,9.80665,0.0))
	    EndIf	
	    '  Apply gravity To all other balls.
	    For i = 1 To Balls-1
	        iBodyAccelerationApply(Ball[i],_D3DXVECTOR3(0.0,-9.80665,0.0))
		Next
		
	    ' ************************************************
	    '  MODIFIED For THIS DEMO
	    ' ************************************************
	
	    '  Acquire distance between camera And each ball.
	   ' define some temporary vectors
	    Local cameralocation:D3DXVECTOR3=New D3DXVECTOR3
	    Local balllocation:D3DXVECTOR3=New D3DXVECTOR3

	    iCameraLocation(Camera,cameralocation)
	    For i = 0 To Balls-1
	        iMeshLocation(BallMesh[i],balllocation)
	        BallDist[i] = iVectorLengthSq(balllocation.Minus(cameralocation))
	    Next
	
	    '  Clear node list.
	    For i = 0 To Balls-1
	        Node[i].prev = Null
	        Node[i]._next = Null
	    Next
	
	    '  Add first mesh directly, so following balls have something To compare against.
	    Nodes       = 1
	    first       = Node[0]
	    first.mesh = BallMesh[0]
	    first.dist = BallDist[0]
	
	    '  Add remaining balls by finding the Right place For them in the linked list.
	    '  NOTE: Must be added in reverse order, that is, from farthest To nearest.
	    Local curr:_NODE = first
	    For i = 1 To Balls-1
	
	        Repeat
	
	            If (BallDist[i] > curr.dist)
	                '  This ball is further than the current node.
	
	                If (curr.prev)
	                    '  There is a previous node - jump To it now.
	                    curr = curr.prev
	
	                    '  Check If ball is closer than predeccessor.
	                    If (BallDist[i] < curr.dist)
	                        '  Ball is further than previous node, but closer than its predeccessor - so insert between them.
	                        Node[Nodes].prev = curr
	                        Node[Nodes]._next = curr._next
	                        Node[Nodes].mesh = BallMesh[i]
	                        Node[Nodes].dist = BallDist[i]
	                        curr._next.prev = Node[Nodes]
	                        curr._next       = Node[Nodes]
	                        Nodes:+1
	                        Exit
	                    EndIf
	                    
	                Else
	                    '  This is the first node - add the New node To the start of the list And Proceed To Next ball.
	                    Node[Nodes]._next = curr
	                    Node[Nodes].mesh = BallMesh[i]
	                    Node[Nodes].dist = BallDist[i]
	                    curr.prev = Node[Nodes]
	                    first      = Node[Nodes]
	                    Nodes:+1
	                    Exit
	                EndIf
	
	            Else
	                '  This ball is closer than the current node.
	
	                If (curr._next)
	                    '  There is a successor node - jump To it now.
	                    curr = curr._next
	
	                    '  Check If ball is further than successor.
	                    If (BallDist[i] > curr.dist)
	                        '  Ball is closer than previous node, but furhter than its successor - so insert between them.
	                        Node[Nodes].prev = curr.prev
	                        Node[Nodes]._next = curr
	                        Node[Nodes].mesh = BallMesh[i]
	                        Node[Nodes].dist = BallDist[i]
	                        curr.prev._next = Node[Nodes]
	                        curr.prev       = Node[Nodes]
	                        Nodes:+1
	                        Exit
	                    EndIf
	
	                Else
	                    '  This is the last node - add the New node To the End of the list And Proceed To Next ball.
	                    Node[Nodes].prev = curr
	                    Node[Nodes].mesh = BallMesh[i]
	                    Node[Nodes].dist = BallDist[i]
	                    curr._next = Node[Nodes]
	                    Nodes:+1
	                    Exit
	                EndIf
	            EndIf
	        Forever
	    Next
	
	    '  Set rendering order, from farthest (1) To closest (n).
	    Local order:Int = 1
	    Repeat
	        iMeshRenderPrioritySet(first.mesh,order)
	        first = first._next
	        order:+1
	    Until Not first
	
	    ' ************************************************
	
	    '  Select render mode For the balls according To current Rendermode variable state
	    If (Rendermode = 0)
	        '  Render the balls in normal mode
	        For i = 0 To Balls-1
	            iMeshRenderModeSet(BallMesh[i],D3DBLEND_SRCALPHA,D3DBLEND_INVSRCALPHA,True,False)
		   Next
	    Else
	        '  Render the balls in 'burnout' mode
	        For i = 0 To Balls-1
	            iMeshRenderModeSet(BallMesh[i],D3DBLEND_SRCALPHA,D3DBLEND_ONE,False,False)
		   Next
	    EndIf
	    '  Switch render modes on [space] key press
	    If (iKeyClick(DIK_SPACE)) Then Rendermode = Not RenderMode ' If 0 turns 1, If 1 turns 0
	    iPrintCentered("PRESS [SPACE] TO SWITCH RENDERING MODE",_D3DXVECTOR2(0.0,10.0),_D3DXVECTOR2(0.4,0.35),0.1,FontWhite,0.0)
	EndMethod
EndType

