' Bump-mapping demo.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY
Global FontWhite:FONT

Global Texture:TEXTURE
Global RenderSpecular:Int

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()

	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   ' This example is basically the same as the Default BallGame example, except
	   ' that we apply a bump-map texture To the ball mesh with the Next line.
	   ' Note that, because the bump-map texture is a height-map, we must specify
	   ' True as 3rd parameter, To have it converted To a normal-map by the engine.
	   BallMesh=iBodyMeshBumpCreate("default_res\\ball.x","default_res\\heightmap2.jpg",True,Ball)
	
	   ' We also change the main ball texture To uniform green, To better show the bumps
	   Texture = iTextureCreate("default_res\\ballgreen.jpg")
	   iMeshTextureSwap(BallMesh,Texture)
	
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	
	   ' Finally we add some bump-mesh specific ambient light...
	   iMeshBumpAmbientColorSet(BallMesh,0.1,0.2,0.2)
	   ' ...And initialize the specular mode To ON.
	   iMeshSpecularSet(BallMesh,1.0,50.0)
	   RenderSpecular = True
	EndMethod
	
	Method _Run()
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else 
		iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   ' Switch specular mode on [Space] key press
	   If (iKeyClick(DIK_SPACE))
	      RenderSpecular = Not RenderSpecular ' If False set To True And vice-versa
	      If (RenderSpecular)
			iMeshSpecularSet(BallMesh,1.0,50.0)
	      Else
			iMeshSpecularSet(BallMesh,0.0,0.0)
		 EndIf
	   EndIf
	
	   iPrintCentered("PRESS [SPACE] TO SWITCH SPECULAR",_D3DXVECTOR2(0.0,11.0),_D3DXVECTOR2(0.5,0.5),0.1,FontWhite,0.0)
	EndMethod
EndType