' Cameraman avoiding solid scenery features While chasing a keyboard controlled ball.
' Controls: arrow keys, [Ctrl]

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

' In order To use the Cameraman extension, you must Include
' the Cameraman header file with the following line.
Include "..\common\3impact_cameraman.bmx"

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY

' Before you can create a cameraman And call all its functions, you
' must 'define' it and assign a name to it. With the following line
' we define a cameraman called CMKD. The Type of cameraman is specified
' by 'CameramanLookAtPointKeepDistance_t', which means that the cameraman
' will follow a moving target And will keep a constant distance from it.
' There are currently four types (classes) of cameramen defined in the
' 3impact_cameraman.h header file. Refer To that file For more information.
Global CMKD:CameramanLookAtPointKeepDistance_t=New CameramanLookAtPointKeepDistance_t

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()

	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   ' A cameraman must be created at initialization time. You can do it with just one
	   ' single Function call like For example
	   ' CMKD.Create(Camera,1.0,15.0,-1.0,20.0,0.9,0.35,Null)
	   ' but If you want To prevent the cameraman To pass through solid objects, you must
	   ' first allocate a temporary array of BODY objects To pass To the Create() function.
	   ' For example, To allocate a 10 element array of bodies use the following line:
	   Local collisionbodies:BODY[10]
	   ' Then fill the array with the list of bodies you want To be 'solid' for the cameraman.
	   ' Just make sure that the array element that follows the last body is set To NULL.
	   ' In this example we have one single body that we want To be solid For the cameraman.
	   ' It is the floating island (Terrain).
	   collisionbodies[0] = Terrain
	   collisionbodies[1] = Null
	   ' After that you can call the Create() Function For the cameraman And pass the
	   ' array of solid bodies as last parameter.
	   ' Refer To 3impact_cameraman.h To know the meaning of each parameter required by
	   ' the Create() Function, For the CameramanLookAtPointKeepDistance_t cameraman type.
	   CMKD.Create(Camera,1.0,15.0,-1.0,20.0,0.9,0.35,collisionbodies)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	EndMethod
	Method _Run()
	
	   Local actorlocation:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,actorlocation)
	
	   ' At every DLL loop you should call the Update() Function For the cameraman,
	   ' passing the actor location vector as parameter. This will update the cameraman
	   ' location And perform the desired 'chasing' effect.
	   CMKD.Update(actorlocation)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	EndMethod
	Method _Exit()
	   ' When calling the Create() Function For a cameraman, some system memory is
	   ' allocated. You must Release it when the cameraman is no longer needed
	   ' by calling the Destroy() Function For it.
	   CMKD.Destroy()
	EndMethod
EndType

