' Mouse cursor demo

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global FontWhite:FONT

' The mouse cursor is rendered as a sprite. Let's define an object variable for it.
Global Cursor:SPRITE

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   ' Load the mouse cursor sprite file
	   Cursor=iSpriteCreate("default_res\\sprites\\cursor.x",Null)
	
	   ' Set cursor sprite priority so that it is rendered over any text
	   iSpritePrioritySet(Cursor,1)
	
	   ' Hide the Windows mouse pointer so the user doesn't get confused when the
	   ' engine runs in windowed mode. This Function call has no effect If the
	   ' engine runs in full-screen mode, as there is no Windows mouse pointer there.
	   iWindowsMouseHide(True)
	
	   iSpriteHide(LoadingMessage)
	
	   ' Show the mouse cursor sprite
	   iSpriteShow(Cursor)
	EndMethod

	Method _Run()
	   Local cursorlocation:D3DXVECTOR2=_D3DXVECTOR2()
	
	   ' A sprite is at Upper/Left corner of the screen when its coordinates are (-16.0,12.0).
	   ' And it is at Lower/Right corner when its coordinates are (16.0,-12.0).
	   ' However, mouse coordinates are (0.0,0.0) when the mouse is completely pushed To
	   ' the top/Left screen corner, And close To (1.0,1.0) when it is completely pushed To the
	   ' Lower/Right corner. As we want To place the sprite where the current mouse coordinates
	   ' are, we need the following expressions To convert mouse coordinates To sprite coordinates:
	   cursorlocation.x = iMouseX()*32.0-16.0
	   cursorlocation.y = 12.0-iMouseY()*24.0
	   ' Now that we have the sprite coordinates computed And saved To the 'cursorlocation'
	   ' variable (2d vector), we can place the sprite there.
	   iSpriteSet(Cursor,cursorlocation,0.0,_D3DXVECTOR2(1.0,1.0))
	
	   ' Print some statistics To help the user understand how the mouse functions work
	   iPrintCentered("Mouse cursor location: "+cursorlocation.x+","+cursorlocation.y,_D3DXVECTOR2(0.0,2.0),_D3DXVECTOR2(1.0,1.0),0.1,FontWhite,0.0)
	   iPrintCentered("iMouseX(),iMouseY() output: "+iMouseX()+","+iMouseY(),_D3DXVECTOR2(0.0,0.0),_D3DXVECTOR2(1.0,1.0),0.1,FontWhite,0.0)
	   ' mouse wheel...
	   iPrintCentered("Mouse wheel: "+iMouseZ(False),_D3DXVECTOR2(0.0,-2.0),_D3DXVECTOR2(1.0,1.0),0.1,FontWhite,0.0)
	EndMethod
EndType