' Simulation-consistent emission of particles

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY
Global FontWhite:FONT

Global ParticleDust:PARTICLE
Global EmitterDust:EMITTER

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   ' In order To emit the dust at proper time, location And intensity,
	   ' we need To know when a collision happens, where it happens, And what is the intensity
	   ' of the force applied To the ball as a consequence of the collision.
	
	   ' So we must enable collision feedback For the BallTerrain collision couple (bodybody).
	   iBodyBodyFeedbackEnable(BallTerrain,-1,0,True)
	
	   ' Create And configure the particle And an emitter For it
	   ParticleDust=iParticleCreate("default_res\\particle_dust.dds",3000)
	   iParticleRenderModeSet(ParticleDust,0)
	   iParticleGravitySet(ParticleDust,_D3DXVECTOR3(0.0,0.4,0.0))
	   iParticleAirResistanceSet(ParticleDust,-1.0)
	   iParticleColorSet(ParticleDust,_D3DXVECTOR3(0.84,0.80,0.50),_D3DXVECTOR3(0.83,0.80,0.70))
	   iParticleLifeTimeSet(ParticleDust,1.0)
	   EmitterDust=iParticleEmitterCreate(ParticleDust)
	   iParticleEmitterDirectionSet(EmitterDust,_D3DXVECTOR3(-1.0,-1.0,-1.0),_D3DXVECTOR3(1.0,1.0,1.0))
	
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	   iParticleShow(ParticleDust)
	EndMethod
	Method _Run()

	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If Not iKeyDown(DIK_LCONTROL)
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   iPrintCentered("[Ctrl] to bounce",_D3DXVECTOR2(0.0,-11.5),_D3DXVECTOR2(0.5,0.5),0.1,FontWhite,0.0)
	
	   ' Defines a couple of Local variables (vectors) required by the iBodyBodyFeedback() Function
	   Local location:D3DXVECTOR3=_D3DXVECTOR3(),force:D3DXVECTOR3=_D3DXVECTOR3()
	
	   ' The Next line is executed again And again, in the Run() callback, 75 times per second.
	   ' It basically does nothing, unless a collision between the ball And the scenery is
	   ' reported by the iBodyBodyFeedback() function. On a collision, the Function returns
	   ' True, And the If() section is executed.
	   If (iBodyBodyFeedback(BallTerrain,-1,0,location,Null,force,Null,Null,Null))
	      ' If a collision happened during the current dll-loop, this program section is executed.
	
	      ' The vector named 'location' has been filled with the collision location by the
	      ' iBodyBodyFeedback() Function above.
	      ' The vector named 'force' has been filled with the force applied to the ball
	      ' as a consequence of the collision.
	
	      ' The Next line of code basically converts the intensity of the force generated
	      ' by the collision To a value between 0.0f And 1.0f. It puts zero in the
	      ' 'forceintensity' variable when the length (intensity) of 'force' is 1.28f.
	      ' It puts 1.0f when the intensity is 400.0f.
	      ' Note that 'forceintensity' is less than zero when the intensity is less than 3.0,
	      ' And it is greater than 1.0f when the intensity is bigger than 300.0f.
	      Local forceintensity:Float=iFloatInterpolate(iVectorLength(force),1.28,400.0,0.0,1.0,False)
	
	      ' The following line checks If 'forceintensity' is greater than zero. If so the
	      ' subsequent {...} section is executed.
	      If (forceintensity>0.0)
	         ' Places the particle emitter at the same location as the collision point, as
	         ' reported in the 'location' variable by the iBodyBodyFeedback() function above.
	         iParticleEmitterLocationSet(EmitterDust,location)
	
	         ' We want the particle size To be proportional To collision force intensity.
	
	         ' The following line sets 'scalestart' to 1.0f when 'forceintensity' is 0.0,
	         ' And To 2.0f when it is 1.0f.
	         ' As a result, when the ball hits the ground harder, the initial size of each
	         ' particle is bigger.
	         Local scalestart:Float = iFloatInterpolate(forceintensity,0.0,1.0,1.0,2.0,False)
	         ' The following line sets 'scaleend' to 1.5f when 'forceintensity' is 0.0,
	         ' And To 6.0f when it is 1.0f.
	         ' As a result, when the ball hits the ground harder, the Final size of each
	         ' particle is bigger.
	         Local scaleend:Float = iFloatInterpolate(forceintensity,0.0,1.0,1.5,6.0,False)
	         ' Sets the computed initial And Final sizes For the particles that are To be emitted
	         iParticleEmitterScaleSet(EmitterDust,_D3DXVECTOR2(scalestart,scalestart),_D3DXVECTOR2(scaleend,scaleend))
	
	         ' We also want the number of emitted particles per second And their
	         ' opacity To be proportional To collision force intensity.
	
	         ' The following line sets 'frequency' to 75.0f particles per second
	         ' when 'forceintensity' is 0.0, and to 150.0f particles per second when it is 1.0f.
	         ' As a result, when the ball hits the ground harder, the dust cloud looks denser.
	         ' NOTE: because dll-loop frequency is 75 per second, setting the minimum
	         ' emitted particles per second To 75 as well ensures that at least one
	         ' single particle is emitted at every ball-terrain contact.
	         Local frequency:Float = iFloatInterpolate(forceintensity,0.0,1.0,150.0,300.0,False)
	         ' The following line sets 'opacity' to 0.25f (25%) when 'forceintensity' is 0.0,
	         ' And To 1.0f (100%) when it is 1.0f.
	         ' As a result, when the ball hits the ground harder, the dust cloud looks thicker.
	         Local opacity:Float = iFloatInterpolate(forceintensity,0.0,1.0,0.25,1.0,False)
	         ' Emits some dust particles with computed frequency And opacity
	         iParticleEmitterEmit(EmitterDust,frequency,opacity)
	      EndIf
	   EndIf
	EndMethod
EndType

