' Pre-rendering callback demo.

' How To render the same mesh by using different settings For differnt cameras.

' NOTE: in our code comments we assume you understand the SimultaneousViews sample code.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global CameraUpper:CAMERA
Global CameraLower:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY

Global BallLocation:D3DXVECTOR3=_D3DXVECTOR3()

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   ' Create the Upper rendering window
	   CameraUpper=iCameraCreate(0.0,0.0,1.0,0.5)
	   iCameraLocationSet(CameraUpper,_D3DXVECTOR3(0.0,3.0,-10.0))
	   ' Tell the engine To call the PreRender() callback
	   ' before rendering the scene For this camera
	   iCameraPreRenderCallbackEnable(CameraUpper,True)
	
	   ' Create the Lower rendering window
	   CameraLower=iCameraCreate(0.0,0.5,1.0,0.5)
	   iCameraLocationSet(CameraLower,_D3DXVECTOR3(5.0,1.5,-5.0))
	   ' Tell the engine To call the PreRender() callback
	   ' before rendering the scene For this camera
	   iCameraPreRenderCallbackEnable(CameraLower,True)
	
	   ' Render the loading message into both rendering windows
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	EndMethod
	
	Method _Run()

	   iBodyLocationCM(Ball,BallLocation)
	
	   iCameraLookAt(CameraUpper,BallLocation,0.1)
	   iCameraLookAt(CameraLower,BallLocation,0.1)
	
	   Local acceleration:D3DXVECTOR3=_D3DXVECTOR3()
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(CameraUpper,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If (Not iKeyDown(DIK_LCONTROL))
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	EndMethod

	Method _PreRender(camera:CAMERA)
	   ' Here is the PreRender callback.
	
	   ' The engine will call this procedure before rendering
	   ' the scene For each camera. So, For example, If you have
	   ' enabled pre-rendering callback For two cameras, this
	   ' callback is executed two times per DLL loop.
	
	   ' The engine also puts the current camera handle in the 'camera' variable.
	   If (camera.IsEq(CameraUpper))
	      ' The 'camera' variable is identical to CameraUpper handle.
	      ' It means that the Next rendering is For the Upper camera.
	      ' So, set wireframe as rendering mode For the ball.
	      iMeshWireframeEnable(BallMesh,True)
	   EndIf
	
	   If (camera.IsEq(CameraLower))
	      ' The 'camera' variable is identical to CameraLower handle.
	      ' It means that the Next rendering is For the Lower camera.
	      ' So, disable wireframe rendering For the ball.
	      iMeshWireframeEnable(BallMesh,False)
	   EndIf
	EndMethod
EndType

