' Ragdoll demo.

' A ragdoll is made of rigid body parts connected together by ball-Type joints.
' Joint rotations about Local XYZ axes are limited To resemble real human body joints.

' In this example, a rigid mesh is attached To each body (body part) To visualize it.

' NOTE: in our code comments we assume you understand the SkinMeshBoneControl sample code.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH

' For each body part we need a BODY* variable And two MESH* variables

Global Head:BODY
Global HeadMesh:MESH
Global HeadMeshShadow:MESH

Global Chest:BODY
Global ChestMesh:MESH
Global ChestMeshShadow:MESH

Global Hip:BODY
Global HipMesh:MESH
Global HipMeshShadow:MESH

Global UpperArmLeft:BODY
Global UpperArmLeftMesh:MESH
Global UpperArmLeftMeshShadow:MESH

Global UpperArmRight:BODY
Global UpperArmRightMesh:MESH
Global UpperArmRightMeshShadow:MESH

Global ForearmLeft:BODY
Global ForearmLeftMesh:MESH
Global ForearmLeftMeshShadow:MESH

Global ForearmRight:BODY
Global ForearmRightMesh:MESH
Global ForearmRightMeshShadow:MESH

Global ThighLeft:BODY
Global ThighLeftMesh:MESH
Global ThighLeftMeshShadow:MESH

Global ThighRight:BODY
Global ThighRightMesh:MESH
Global ThighRightMeshShadow:MESH

Global ShinLeft:BODY
Global ShinLeftMesh:MESH
Global ShinLeftMeshShadow:MESH

Global ShinRight:BODY
Global ShinRightMesh:MESH
Global ShinRightMeshShadow:MESH

' For each body part we need a BODYBODY* variable

Global HeadTerrain:BODYBODY
Global ChestTerrain:BODYBODY
Global HipTerrain:BODYBODY
Global UpperArmLeftTerrain:BODYBODY
Global UpperArmRightTerrain:BODYBODY
Global ForearmLeftTerrain:BODYBODY
Global ForearmRightTerrain:BODYBODY
Global ThighLeftTerrain:BODYBODY
Global ThighRightTerrain:BODYBODY
Global ShinLeftTerrain:BODYBODY
Global ShinRightTerrain:BODYBODY

' For each body part we need a JOINT* variable

Global HeadBoneJoint:JOINT
Global ChestBoneJoint:JOINT
Global UpperArmLeftBoneJoint:JOINT
Global UpperArmRightBoneJoint:JOINT
Global ForearmLeftBoneJoint:JOINT
Global ForearmRightBoneJoint:JOINT
Global ThighLeftBoneJoint:JOINT
Global ThighRightBoneJoint:JOINT
Global ShinLeftBoneJoint:JOINT
Global ShinRightBoneJoint:JOINT

' We also need some variables For temporary objects

Global SkinMesh:SKINMESH

Global HeadBone:SKINMESHBONE
Global ChestBone:SKINMESHBONE
Global HipBone:SKINMESHBONE
Global UpperArmLeftBone:SKINMESHBONE
Global UpperArmRightBone:SKINMESHBONE
Global ForearmLeftBone:SKINMESHBONE
Global ForearmRightBone:SKINMESHBONE
Global ThighLeftBone:SKINMESHBONE
Global ThighRightBone:SKINMESHBONE
Global ShinLeftBone:SKINMESHBONE
Global ShinRightBone:SKINMESHBONE

Global HeadBoneLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global ChestBoneLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global HipBoneLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global UpperArmLeftBoneLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global UpperArmRightBoneLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global ForearmLeftBoneLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global ForearmRightBoneLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global ThighLeftBoneLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global ThighRightBoneLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global ShinLeftBoneLocation:D3DXVECTOR3=_D3DXVECTOR3()
Global ShinRightBoneLocation:D3DXVECTOR3=_D3DXVECTOR3()

Global HeadBoneOrientation:D3DXQUATERNION=_D3DXQUATERNION()
Global ChestBoneOrientation:D3DXQUATERNION=_D3DXQUATERNION()
Global HipBoneOrientation:D3DXQUATERNION=_D3DXQUATERNION()
Global UpperArmLeftBoneOrientation:D3DXQUATERNION=_D3DXQUATERNION()
Global UpperArmRightBoneOrientation:D3DXQUATERNION=_D3DXQUATERNION()
Global ForearmLeftBoneOrientation:D3DXQUATERNION=_D3DXQUATERNION()
Global ForearmRightBoneOrientation:D3DXQUATERNION=_D3DXQUATERNION()
Global ThighLeftBoneOrientation:D3DXQUATERNION=_D3DXQUATERNION()
Global ThighRightBoneOrientation:D3DXQUATERNION=_D3DXQUATERNION()
Global ShinLeftBoneOrientation:D3DXQUATERNION=_D3DXQUATERNION()
Global ShinRightBoneOrientation:D3DXQUATERNION=_D3DXQUATERNION()

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,1.0,-3.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   ' Set initial ragdoll location And physics parameters
	
	   Local actorlocation:D3DXVECTOR3 = _D3DXVECTOR3(0.0,1.0,0.0)
	   Local density:Float = 0.13 ' mass density
	   Local ldamp:Float = 0.999 '  linear damping
	   Local adamp:Float = 0.97 ' angular damping
	
	   ' For each body part create:
	   ' - a dynamic body (from .spg resource)
	   ' - a mesh (from .x 3d texture-mapped model)
	   ' - a shadow-caster mesh (from .x 3d model)
	   ' Also set initial location And damping
	
	   Head=iBodySGCreate("default_res\\ragdoll\\androHead_.spg",density)
	   HeadMesh=iBodyMeshCreate("default_res\\ragdoll\\androHead.x",Head)
	   HeadMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ragdoll\\androHead_shadow.x",Head,50.0)
	   iBodyLocationSet(Head,actorlocation,True)
	   iBodyDampingSet(Head,0.0,ldamp,0.0,adamp)
	   iBodyDamping2Set(Head,100.0,0.05,3600.0,0.05)
	
	   Chest=iBodySGCreate("default_res\\ragdoll\\androChest_.spg",density)
	   ChestMesh=iBodyMeshCreate("default_res\\ragdoll\\androChest.x",Chest)
	   ChestMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ragdoll\\androChest_shadow.x",Chest,50.0)
	   iBodyLocationSet(Chest,actorlocation,True)
	   iBodyDampingSet(Chest,0.0,ldamp,0.0,adamp)
	   iBodyDamping2Set(Chest,100.0,0.05,3600.0,0.05)
	
	   Hip=iBodySGCreate("default_res\\ragdoll\\androHip_.spg",density)
	   HipMesh=iBodyMeshCreate("default_res\\ragdoll\\androHip.x",Hip)
	   HipMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ragdoll\\androHip_shadow.x",Hip,50.0)
	   iBodyLocationSet(Hip,actorlocation,True)
	   iBodyDampingSet(Hip,0.0,ldamp,0.0,adamp)
	   iBodyDamping2Set(Hip,100.0,0.05,3600.0,0.05)
	
	   UpperArmLeft=iBodySGCreate("default_res\\ragdoll\\androUpperArmLeft_.spg",density)
	   UpperArmLeftMesh=iBodyMeshCreate("default_res\\ragdoll\\androUpperArmLeft.x",UpperArmLeft)
	   UpperArmLeftMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ragdoll\\androUpperArmLeft_shadow.x",UpperArmLeft,50.0)
	   iBodyLocationSet(UpperArmLeft,actorlocation,True)
	   iBodyDampingSet(UpperArmLeft,0.0,ldamp,0.0,adamp)
	   iBodyDamping2Set(UpperArmLeft,100.0,0.05,3600.0,0.05)
	
	   UpperArmRight=iBodySGCreate("default_res\\ragdoll\\androUpperArmRight_.spg",density)
	   UpperArmRightMesh=iBodyMeshCreate("default_res\\ragdoll\\androUpperArmRight.x",UpperArmRight)
	   UpperArmRightMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ragdoll\\androUpperArmRight_shadow.x",UpperArmRight,50.0)
	   iBodyLocationSet(UpperArmRight,actorlocation,True)
	   iBodyDampingSet(UpperArmRight,0.0,ldamp,0.0,adamp)
	   iBodyDamping2Set(UpperArmRight,100.0,0.05,3600.0,0.05)
	
	   ForearmLeft=iBodySGCreate("default_res\\ragdoll\\androForearmLeft_.spg",density)
	   ForearmLeftMesh=iBodyMeshCreate("default_res\\ragdoll\\androForearmLeft.x",ForearmLeft)
	   ForearmLeftMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ragdoll\\androForearmLeft_shadow.x",ForearmLeft,50.0)
	   iBodyLocationSet(ForearmLeft,actorlocation,True)
	   iBodyDampingSet(ForearmLeft,0.0,ldamp,0.0,adamp)
	   iBodyDamping2Set(ForearmLeft,100.0,0.05,3600.0,0.05)
	
	   ForearmRight=iBodySGCreate("default_res\\ragdoll\\androForearmRight_.spg",density)
	   ForearmRightMesh=iBodyMeshCreate("default_res\\ragdoll\\androForearmRight.x",ForearmRight)
	   ForearmRightMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ragdoll\\androForearmRight_shadow.x",ForearmRight,50.0)
	   iBodyLocationSet(ForearmRight,actorlocation,True)
	   iBodyDampingSet(ForearmRight,0.0,ldamp,0.0,adamp)
	   iBodyDamping2Set(ForearmRight,100.0,0.05,3600.0,0.05)
	
	   ThighLeft=iBodySGCreate("default_res\\ragdoll\\androThighLeft_.spg",density)
	   ThighLeftMesh=iBodyMeshCreate("default_res\\ragdoll\\androThighLeft.x",ThighLeft)
	   ThighLeftMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ragdoll\\androThighLeft_shadow.x",ThighLeft,50.0)
	   iBodyLocationSet(ThighLeft,actorlocation,True)
	   iBodyDampingSet(ThighLeft,0.0,ldamp,0.0,adamp)
	   iBodyDamping2Set(ThighLeft,100.0,0.05,3600.0,0.05)
	
	   ThighRight=iBodySGCreate("default_res\\ragdoll\\androThighRight_.spg",density)
	   ThighRightMesh=iBodyMeshCreate("default_res\\ragdoll\\androThighRight.x",ThighRight)
	   ThighRightMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ragdoll\\androThighRight_shadow.x",ThighRight,50.0)
	   iBodyLocationSet(ThighRight,actorlocation,True)
	   iBodyDampingSet(ThighRight,0.0,ldamp,0.0,adamp)
	   iBodyDamping2Set(ThighRight,100.0,0.05,3600.0,0.05)
	
	   ShinLeft=iBodySGCreate("default_res\\ragdoll\\androShinLeft_.spg",density)
	   ShinLeftMesh=iBodyMeshCreate("default_res\\ragdoll\\androShinLeft.x",ShinLeft)
	   ShinLeftMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ragdoll\\androShinLeft_shadow.x",ShinLeft,50.0)
	   iBodyLocationSet(ShinLeft,actorlocation,True)
	   iBodyDampingSet(ShinLeft,0.0,ldamp,0.0,adamp)
	   iBodyDamping2Set(ShinLeft,100.0,0.05,3600.0,0.05)
	
	   ShinRight=iBodySGCreate("default_res\\ragdoll\\androShinRight_.spg",density)
	   ShinRightMesh=iBodyMeshCreate("default_res\\ragdoll\\androShinRight.x",ShinRight)
	   ShinRightMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ragdoll\\androShinRight_shadow.x",ShinRight,50.0)
	   iBodyLocationSet(ShinRight,actorlocation,True)
	   iBodyDampingSet(ShinRight,0.0,ldamp,0.0,adamp)
	   iBodyDamping2Set(ShinRight,100.0,0.05,3600.0,0.05)
	
	   ' We use bone locations/orientations from a skinned 3d model To
	   ' initialize the joints that will link all body parts together.
	
	   ' Create a skinmesh from a boned .x model And set its location
	   ' so that it matches the dynamic bodies (body parts).
	
	   SkinMesh=iSkinMeshCreate("default_res\\ragdoll\\androrest.x",Null)
	   iSkinMeshLocationSet(SkinMesh,actorlocation)
	
	   ' Create skinmeshbone objects For some bones To be able
	   ' To retrieve their location And orientation later
	
	   HeadBone=iSkinMeshBoneCreateByName(SkinMesh,"HeadBone")
	   ChestBone=iSkinMeshBoneCreateByName(SkinMesh,"ChestBone")
	   HipBone=iSkinMeshBoneCreateByName(SkinMesh,"HipBone")
	   UpperArmLeftBone=iSkinMeshBoneCreateByName(SkinMesh,"UpperArmLeftBone")
	   UpperArmRightBone=iSkinMeshBoneCreateByName(SkinMesh,"UpperArmRightBone")
	   ForearmLeftBone=iSkinMeshBoneCreateByName(SkinMesh,"ForearmLeftBone")
	   ForearmRightBone=iSkinMeshBoneCreateByName(SkinMesh,"ForearmRightBone")
	   ThighLeftBone=iSkinMeshBoneCreateByName(SkinMesh,"ThighLeftBone")
	   ThighRightBone=iSkinMeshBoneCreateByName(SkinMesh,"ThighRightBone")
	   ShinLeftBone=iSkinMeshBoneCreateByName(SkinMesh,"ShinLeftBone")
	   ShinRightBone=iSkinMeshBoneCreateByName(SkinMesh,"ShinRightBone")
	
	   ' Retrieve And store each bone's location
	
	   iSkinMeshBoneLocation(HeadBone,HeadBoneLocation,0)
	   iSkinMeshBoneLocation(ChestBone,ChestBoneLocation,0)
	   iSkinMeshBoneLocation(UpperArmLeftBone,UpperArmLeftBoneLocation,0)
	   iSkinMeshBoneLocation(UpperArmRightBone,UpperArmRightBoneLocation,0)
	   iSkinMeshBoneLocation(ForearmLeftBone,ForearmLeftBoneLocation,0)
	   iSkinMeshBoneLocation(ForearmRightBone,ForearmRightBoneLocation,0)
	   iSkinMeshBoneLocation(ThighLeftBone,ThighLeftBoneLocation,0)
	   iSkinMeshBoneLocation(ThighRightBone,ThighRightBoneLocation,0)
	   iSkinMeshBoneLocation(ShinLeftBone,ShinLeftBoneLocation,0)
	   iSkinMeshBoneLocation(ShinRightBone,ShinRightBoneLocation,0)
	
	   ' Retrieve And store each bone's orientation
	
	   iSkinMeshBoneOrientation(HeadBone,HeadBoneOrientation,0)
	   iSkinMeshBoneOrientation(ChestBone,ChestBoneOrientation,0)
	   iSkinMeshBoneOrientation(UpperArmLeftBone,UpperArmLeftBoneOrientation,0)
	   iSkinMeshBoneOrientation(UpperArmRightBone,UpperArmRightBoneOrientation,0)
	   iSkinMeshBoneOrientation(ForearmLeftBone,ForearmLeftBoneOrientation,0)
	   iSkinMeshBoneOrientation(ForearmRightBone,ForearmRightBoneOrientation,0)
	   iSkinMeshBoneOrientation(ThighLeftBone,ThighLeftBoneOrientation,0)
	   iSkinMeshBoneOrientation(ThighRightBone,ThighRightBoneOrientation,0)
	   iSkinMeshBoneOrientation(ShinLeftBone,ShinLeftBoneOrientation,0)
	   iSkinMeshBoneOrientation(ShinRightBone,ShinRightBoneOrientation,0)
	
	   ' Skinmeshbone objects And the reference skinmesh are no longer needed now
		' in BMAX we set to Null.  this in turn calls the proper 3Impact destroy function when the GC cleans up.		
	   ShinRightBone=Null
	   ShinLeftBone=Null
	   ThighRightBone=Null
	   ThighLeftBone=Null
	   ForearmRightBone=Null
	   ForearmLeftBone=Null
	   UpperArmRightBone=Null
	   UpperArmLeftBone=Null
	   HipBone=Null
	   ChestBone=Null
	   HeadBone=Null
	   SkinMesh=Null
	
	   ' Create And position joints by using retrieved bone locations
	
	   HeadBoneJoint = iJointBallCreate(Chest,Head,HeadBoneLocation)
	   ChestBoneJoint = iJointBallCreate(Hip,Chest,ChestBoneLocation)
	   UpperArmLeftBoneJoint = iJointBallCreate(Chest,UpperArmLeft,UpperArmLeftBoneLocation)
	   UpperArmRightBoneJoint = iJointBallCreate(Chest,UpperArmRight,UpperArmRightBoneLocation)
	   ForearmLeftBoneJoint = iJointBallCreate(UpperArmLeft,ForearmLeft,ForearmLeftBoneLocation)
	   ForearmRightBoneJoint = iJointBallCreate(UpperArmRight,ForearmRight,ForearmRightBoneLocation)
	   ThighLeftBoneJoint = iJointBallCreate(Hip,ThighLeft,ThighLeftBoneLocation)
	   ThighRightBoneJoint = iJointBallCreate(Hip,ThighRight,ThighRightBoneLocation)
	   ShinLeftBoneJoint = iJointBallCreate(ThighLeft,ShinLeft,ShinLeftBoneLocation)
	   ShinRightBoneJoint = iJointBallCreate(ThighRight,ShinRight,ShinRightBoneLocation)
	
	   ' Orientate joints by using retrieved bone orientations
	
	   iJointBallOrientationSet(HeadBoneJoint,HeadBoneOrientation)
	   iJointBallOrientationSet(ChestBoneJoint,ChestBoneOrientation)
	   iJointBallOrientationSet(UpperArmLeftBoneJoint,UpperArmLeftBoneOrientation)
	   iJointBallOrientationSet(UpperArmRightBoneJoint,UpperArmRightBoneOrientation)
	   iJointBallOrientationSet(ForearmLeftBoneJoint,ForearmLeftBoneOrientation)
	   iJointBallOrientationSet(ForearmRightBoneJoint,ForearmRightBoneOrientation)
	   iJointBallOrientationSet(ThighLeftBoneJoint,ThighLeftBoneOrientation)
	   iJointBallOrientationSet(ThighRightBoneJoint,ThighRightBoneOrientation)
	   iJointBallOrientationSet(ShinLeftBoneJoint,ShinLeftBoneOrientation)
	   iJointBallOrientationSet(ShinRightBoneJoint,ShinRightBoneOrientation)
	
	   ' For each joint, set proper rotation limits, limit characteristics And also
	   ' simulate some 'rotational friction' by setting a 'zero-velocity' angular motor
	
	   iJointBallXStopsSet(HeadBoneJoint,-35.0,35.0,25.0,1.0)
	   iJointBallYStopsSet(HeadBoneJoint,-15.0,15.0,25.0,1.0)
	   iJointBallZStopsSet(HeadBoneJoint,-20.0,20.0,25.0,1.0)
	   iJointBallXAMotorSet(HeadBoneJoint,0.0,0.02,1.0)
	   iJointBallYAMotorSet(HeadBoneJoint,0.0,0.02,1.0)
	   iJointBallZAMotorSet(HeadBoneJoint,0.0,0.02,1.0)
	
	   iJointBallXStopsSet(ChestBoneJoint,-15.0,15.0,25.0,1.0)
	   iJointBallYStopsSet(ChestBoneJoint,-10.0,10.0,25.0,1.0)
	   iJointBallZStopsSet(ChestBoneJoint,-5.0,5.0,25.0,1.0)
	   iJointBallXAMotorSet(ChestBoneJoint,0.0,0.1,1.0)
	   iJointBallYAMotorSet(ChestBoneJoint,0.0,0.1,1.0)
	   iJointBallZAMotorSet(ChestBoneJoint,0.0,0.1,1.0)
	
	   iJointBallXStopsSet(UpperArmLeftBoneJoint,-10.0,70.0,25.0,1.0)
	   iJointBallYStopsSet(UpperArmLeftBoneJoint,-70.0,10.0,25.0,1.0)
	   iJointBallZStopsSet(UpperArmLeftBoneJoint,-5.0,5.0,25.0,1.0)
	   iJointBallXAMotorSet(UpperArmLeftBoneJoint,0.0,0.02,1.0)
	   iJointBallYAMotorSet(UpperArmLeftBoneJoint,0.0,0.02,1.0)
	   iJointBallZAMotorSet(UpperArmLeftBoneJoint,0.0,0.02,1.0)
	
	   iJointBallXStopsSet(UpperArmRightBoneJoint,-10.0,70.0,25.0,1.0)
	   iJointBallYStopsSet(UpperArmRightBoneJoint,-10.0,70.0,25.0,1.0)
	   iJointBallZStopsSet(UpperArmRightBoneJoint,-5.0,5.0,25.0,1.0)
	   iJointBallXAMotorSet(UpperArmRightBoneJoint,0.0,0.02,1.0)
	   iJointBallYAMotorSet(UpperArmRightBoneJoint,0.0,0.02,1.0)
	   iJointBallZAMotorSet(UpperArmRightBoneJoint,0.0,0.02,1.0)
	
	   iJointBallXStopsSet(ForearmLeftBoneJoint,0.0,105.0,25.0,1.0)
	   iJointBallYStopsSet(ForearmLeftBoneJoint,0.0,0.0,25.0,1.0)
	   iJointBallZStopsSet(ForearmLeftBoneJoint,0.0,0.0,25.0,1.0)
	   iJointBallXAMotorSet(ForearmLeftBoneJoint,0.0,0.1,1.0)
	   iJointBallYAMotorSet(ForearmLeftBoneJoint,0.0,0.1,1.0)
	   iJointBallZAMotorSet(ForearmLeftBoneJoint,0.0,0.1,1.0)
	
	   iJointBallXStopsSet(ForearmRightBoneJoint,0.0,105.0,25.0,1.0)
	   iJointBallYStopsSet(ForearmRightBoneJoint,0.0,0.0,25.0,1.0)
	   iJointBallZStopsSet(ForearmRightBoneJoint,0.0,0.0,25.0,1.0)
	   iJointBallXAMotorSet(ForearmRightBoneJoint,0.0,0.1,1.0)
	   iJointBallYAMotorSet(ForearmRightBoneJoint,0.0,0.1,1.0)
	   iJointBallZAMotorSet(ForearmRightBoneJoint,0.0,0.1,1.0)
	
	   iJointBallXStopsSet(ThighLeftBoneJoint,-5.0,80.0,10.0,0.0)
	   iJointBallYStopsSet(ThighLeftBoneJoint,0.0,40.0,25.0,1.0)
	   iJointBallZStopsSet(ThighLeftBoneJoint,0.0,0.0,25.0,1.0)
	   iJointBallXAMotorSet(ThighLeftBoneJoint,0.0,0.01,1.0)
	   iJointBallYAMotorSet(ThighLeftBoneJoint,0.0,0.01,1.0)
	   iJointBallZAMotorSet(ThighLeftBoneJoint,0.0,0.01,1.0)
	
	   iJointBallXStopsSet(ThighRightBoneJoint,-5.0,80.0,10.0,0.0)
	   iJointBallYStopsSet(ThighRightBoneJoint,-40.0,0.0,25.0,1.0)
	   iJointBallZStopsSet(ThighRightBoneJoint,0.0,0.0,25.0,1.0)
	   iJointBallXAMotorSet(ThighRightBoneJoint,0.0,0.01,1.0)
	   iJointBallYAMotorSet(ThighRightBoneJoint,0.0,0.01,1.0)
	   iJointBallZAMotorSet(ThighRightBoneJoint,0.0,0.01,1.0)
	
	   iJointBallXStopsSet(ShinLeftBoneJoint,-105.0,0.0,5.0,0.0)
	   iJointBallYStopsSet(ShinLeftBoneJoint,0.0,0.0,25.0,1.0)
	   iJointBallZStopsSet(ShinLeftBoneJoint,0.0,0.0,25.0,1.0)
	   iJointBallXAMotorSet(ShinLeftBoneJoint,0.0,0.1,1.0)
	   iJointBallYAMotorSet(ShinLeftBoneJoint,0.0,0.1,1.0)
	   iJointBallZAMotorSet(ShinLeftBoneJoint,0.0,0.1,1.0)
	
	   iJointBallXStopsSet(ShinRightBoneJoint,-105.0,0.0,5.0,0.0)
	   iJointBallYStopsSet(ShinRightBoneJoint,0.0,0.0,25.0,1.0)
	   iJointBallZStopsSet(ShinRightBoneJoint,0.0,0.0,25.0,1.0)
	   iJointBallXAMotorSet(ShinRightBoneJoint,0.0,0.1,1.0)
	   iJointBallYAMotorSet(ShinRightBoneJoint,0.0,0.1,1.0)
	   iJointBallZAMotorSet(ShinRightBoneJoint,0.0,0.1,1.0)
	
	   ' Enable collision detection For each body part with the scenery
	
	   HeadTerrain=iBodyBodyCreate(Head,Terrain)
	   ChestTerrain=iBodyBodyCreate(Chest,Terrain)
	   HipTerrain=iBodyBodyCreate(Hip,Terrain)
	   UpperArmLeftTerrain=iBodyBodyCreate(UpperArmLeft,Terrain)
	   UpperArmRightTerrain=iBodyBodyCreate(UpperArmRight,Terrain)
	   ForearmLeftTerrain=iBodyBodyCreate(ForearmLeft,Terrain)
	   ForearmRightTerrain=iBodyBodyCreate(ForearmRight,Terrain)
	   ThighLeftTerrain=iBodyBodyCreate(ThighLeft,Terrain)
	   ThighRightTerrain=iBodyBodyCreate(ThighRight,Terrain)
	   ShinLeftTerrain=iBodyBodyCreate(ShinLeft,Terrain)
	   ShinRightTerrain=iBodyBodyCreate(ShinRight,Terrain)
	
	   ' Set collision response factors
	
	   Local friction:Float = 2.5
	   iBodyBodyFrictionSet(HeadTerrain,friction)
	   iBodyBodyFrictionSet(ChestTerrain,friction)
	   iBodyBodyFrictionSet(HipTerrain,friction)
	   iBodyBodyFrictionSet(UpperArmLeftTerrain,friction)
	   iBodyBodyFrictionSet(UpperArmRightTerrain,friction)
	   iBodyBodyFrictionSet(ForearmLeftTerrain,friction)
	   iBodyBodyFrictionSet(ForearmRightTerrain,friction)
	   iBodyBodyFrictionSet(ThighLeftTerrain,friction)
	   iBodyBodyFrictionSet(ThighRightTerrain,friction)
	   iBodyBodyFrictionSet(ShinLeftTerrain,friction)
	   iBodyBodyFrictionSet(ShinRightTerrain,friction)
	
	   ' Enable all bodies
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Head)
	   iBodyEnable(Chest)
	   iBodyEnable(Hip)
	   iBodyEnable(UpperArmLeft)
	   iBodyEnable(UpperArmRight)
	   iBodyEnable(ForearmLeft)
	   iBodyEnable(ForearmRight)
	   iBodyEnable(ThighLeft)
	   iBodyEnable(ThighRight)
	   iBodyEnable(ShinLeft)
	   iBodyEnable(ShinRight)
	
	   ' Hide 'loading...' message sprite and show all meshes
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	
	   iMeshShow(HeadMesh)
	   iMeshShow(ChestMesh)
	   iMeshShow(HipMesh)
	   iMeshShow(UpperArmLeftMesh)
	   iMeshShow(UpperArmRightMesh)
	   iMeshShow(ForearmLeftMesh)
	   iMeshShow(ForearmRightMesh)
	   iMeshShow(ThighLeftMesh)
	   iMeshShow(ThighRightMesh)
	   iMeshShow(ShinLeftMesh)
	   iMeshShow(ShinRightMesh)
	
	   iMeshShow(HeadMeshShadow)
	   iMeshShow(ChestMeshShadow)
	   iMeshShow(HipMeshShadow)
	   iMeshShow(UpperArmLeftMeshShadow)
	   iMeshShow(UpperArmRightMeshShadow)
	   iMeshShow(ForearmLeftMeshShadow)
	   iMeshShow(ForearmRightMeshShadow)
	   iMeshShow(ThighLeftMeshShadow)
	   iMeshShow(ThighRightMeshShadow)
	   iMeshShow(ShinLeftMeshShadow)
	   iMeshShow(ShinRightMeshShadow)
	EndMethod
	
	Method _Run()
	   ' We want the camera To look at the ragdoll's hip
	
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Hip,cameratarget)
	   iCameraLookAt(Camera,cameratarget,0.1)
	
	   ' Build an acceleration vector on arrow key presses
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-50.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(50.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,50.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-50.0))
	
	   ' Re-orient the vector so that it is camera relative. This way the user
	   ' will press [Left] To push the ragdoll Left, [Right] To push it Right
	   ' And so on, no matter what the camera orientation is. 
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   ' Apply the acceleration To ragdoll's Chest
	
	   iBodyAccelerationApply(Chest,acceleration)
	
	   ' Apply gravity acceleration To all body parts.
	   ' Also apply some upward acceleration when [Ctrl] key is pressed
	
	   Local gravity:Float = 9.80665
	   iBodyAccelerationApply(Head,_D3DXVECTOR3(0.0,-gravity,0.0))
	   iBodyAccelerationApply(Chest,_D3DXVECTOR3(0.0,-gravity,0.0))
	   iBodyAccelerationApply(Hip,_D3DXVECTOR3(0.0,-gravity,0.0))
	   iBodyAccelerationApply(UpperArmLeft,_D3DXVECTOR3(0.0,-gravity,0.0))
	   iBodyAccelerationApply(UpperArmRight,_D3DXVECTOR3(0.0,-gravity,0.0))
	   iBodyAccelerationApply(ForearmLeft,_D3DXVECTOR3(0.0,-gravity,0.0))
	   iBodyAccelerationApply(ForearmRight,_D3DXVECTOR3(0.0,-gravity,0.0))
	   iBodyAccelerationApply(ThighLeft,_D3DXVECTOR3(0.0,-gravity,0.0))
	   iBodyAccelerationApply(ThighRight,_D3DXVECTOR3(0.0,-gravity,0.0))
	   iBodyAccelerationApply(ShinLeft,_D3DXVECTOR3(0.0,-gravity,0.0))
	   iBodyAccelerationApply(ShinRight,_D3DXVECTOR3(0.0,-gravity,0.0))
	   If (iKeyDown(DIK_LCONTROL) Or iKeyDown(DIK_RCONTROL)) iBodyAccelerationApply(Chest,_D3DXVECTOR3(0.0,gravity*12.0,0.0))
	EndMethod
EndType
