'  Simple keyboard controlled car, with automated track recording.
'  Simple keyboard controlled car, with automated track recording.

'  By Damian Rajkowski (WeirdPumpkin).

'  This example is based on the 'SimpleCar' demo, with modifications to
'  allow recording of the car's manouvers, and playback of those manouvers.
'  The principle of this basic tracking system is recording the time at which the
'  user CHANGES either the car's steering or thrust parameters.
'  This reduces the amount of data considerably, over an alternative like
'  continuously recording the state of the car after a given timeslice.

'  Controls:
'  Turning - Left/Right arrow keys.
'  Forward - up arrow key.
'  Reverse - down arrow key.
'  R - record New track.
'  P - play back last recorded track.
'  S - stop recording/playing.

'  This source code can be freely distributed by any means.

'  THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib
'Import BRL.Stream

'  Declare Object variables
Global Camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global CarBody:BODY
Global CarBodyMesh:MESH
Global CarBodyMeshShadow:MESH
Global CarWheelFL:BODY
Global CarWheelFLMesh:MESH
Global CarWheelFLMeshShadow:MESH
Global CarWheelFR:BODY
Global CarWheelFRMesh:MESH
Global CarWheelFRMeshShadow:MESH
Global CarWheelRL:BODY
Global CarWheelRLMesh:MESH
Global CarWheelRLMeshShadow:MESH
Global CarWheelRR:BODY
Global CarWheelRRMesh:MESH
Global CarWheelRRMeshShadow:MESH
Global CarBodyTerrain:BODYBODY
Global CarWheelFLTerrain:BODYBODY
Global CarWheelFRTerrain:BODYBODY
Global CarWheelRLTerrain:BODYBODY
Global CarWheelRRTerrain:BODYBODY
Global CarWheelFLJoint:WHEEL
Global CarWheelFRJoint:WHEEL
Global CarWheelRLJoint:WHEEL
Global CarWheelRRJoint:WHEEL

'  Declare parameter variables
Global SideFriction:Float
Global RollingWiseFriction:Float
Global SuspensionSpring:Float
Global SuspensionDamping:Float
Global SteerAngle:Float
Global Spin:Float
Global WheelSpin:Float
Global WheelPower:Float

' **********************************************
'  ADDED For THIS DEMO
' **********************************************

'  Added font For advising user when recording Or playing.
Global FontWhite:FONT

'  Flags indicating If car's track is being recorded, or played back.
Global RecordingTrack:Int
Global PlayingTrack:Int

'  Handle For track data file.
Global TrackFile:TStream = Null

'  Timer To keep track of WHEN the car's driving forces change.
Global TrackTime:Float

'  Time value For Next move in Track file - For playback.
Global NextTime:Float

'  Keeps track of user's manipulation of the Steering controls (left/right arrow keys).
'  This variable is set To -1 when the user presses Left, +1 when the user presses Right,
'  And 0 when the User is Not pressing either Left Or Right.
Global UserSteer:Int                      

'  Keeps track of user's manipulation of the Forward/Reverse controls (up/down arrow keys).
'  This variable is set To +1 when the user presses Forward, -1 when Reversing, And
'  0 when the User is Not pressing either Up Or Down.
Global UserDrive:Int

'  Keeps track of user's manipulation of the Brake control (left/right control keys).
'  This variable is set To 1 when the user brakes, 0 when Not braking.
Global UserBrake:Int

'  NOTE: When either UserSteer, UserDrive Or UserBrake changes, a record is written To the
'        track file, specifying the 'timer' value and the specific change. This should reduce
'        the track data considerably, eg. when the car starts a single record will be written
'        (UserDrive 0 To 1) And no further records will be written about UserDrive unless
'        the car stops accelerating (User releases Up arrow key) Or brakes (Down key).

'  NOTE 2: For the purposes of fun, the playback will Not begin at the same spatial point
'          as the recording started. Instead, the car will Repeat the motions the user
'          originally recorded, but from where the car WAS when the playback was started.
'          This means you can record a set of actions, And Then replay Then over And over
'          from various points on the map To see how they play out. Try it!
'          Of course, If you want To implement EXACT playback, simply record the location
'          of the car when the recording starts, And set the car To that position when the
'          playback starts.

'  Stop recording Or playback of track data.
Function iTrackStop()

    '  If Not recording Or playing don't do anything.
    If Not RecordingTrack And Not TrackFile Return

    '  Close track file, first checking If handle is actually used - To prevent errors.
    If (TrackFile) CloseStream(TrackFile)

    '  Reset flags
    RecordingTrack = False
    PlayingTrack = False

EndFunction

'  Write change in car's steering to track file, indicated as 'S'.
Function iTrackWriteSteer(NewValue:Int)

    '  Check If actually recording And track file is valid before writing.
    If Not RecordingTrack Or Not TrackFile Return

    WriteLine(TrackFile,TrackTime)
    WriteLine(TrackFile,"S"+NewValue)
    'fprintf(TrackFile,"%.3f\n",TrackTime)
    'fprintf(TrackFile,"S%i\n",NewValue)

    '  Update variable.
    UserSteer = NewValue

EndFunction

'  Write change in car's drive to track file, indicated as 'D'.
Function iTrackWriteDrive(NewValue:Int)

    '  Check If actually recording And track file is valid before writing.
    If Not RecordingTrack Or Not TrackFile Return

    WriteLine(TrackFile,TrackTime)
    WriteLine(TrackFile,"D"+NewValue)
    'fprintf(TrackFile,"%.3f\n",TrackTime)
    'fprintf(TrackFile,"D%i\n",NewValue)

    '  Update variable.
    UserDrive = NewValue

EndFunction

'  Write change in car's braking to track file, indicated as 'B'.
Function iTrackWriteBrake(NewValue:Int)

    '  Check If actually recording And track file is valid before writing.
    If Not RecordingTrack Or Not TrackFile Return
    
    WriteLine(TrackFile,TrackTime)
    WriteLine(TrackFile,"B"+NewValue)
    'fprintf(TrackFile,"%.3f\n",TrackTime)
    'fprintf(TrackFile,"B%i\n",NewValue)

    '  Update variable.
    UserBrake = NewValue

EndFunction

'  Open a New track data file And begin recording car's motion.
Function iTrackRecord()

    '  If currently playing, stop the playing first.
    If (PlayingTrack) iTrackStop()

    '  Attempt To open track file For writing.
    If FileType("track.dat")=0 Then CreateFile("track.dat")
    TrackFile = OpenFile("track.dat",False,True)

    '  If track file could Not be opened, don't proceed with recording.
    If Not TrackFile Return

    '  Set playing flag And reset timer.
    RecordingTrack = True
    TrackTime = 0.0

EndFunction

'  Play back track manouvers from file.
Function iTrackPlay()

    '  If currently recording, stop the recording first.
    If (RecordingTrack) iTrackStop()

    '  Attempt To open track file For reading.
    TrackFile = OpenFile("track.dat",True,False)

    '  If track file could Not be opened, don't proceed with playing.
    If Not TrackFile Return

    '  Set playing flag And reset timer.
    PlayingTrack = True
    TrackTime = 0.0
    NextTime = 0.0

EndFunction

' **********************************************

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	    ' **********************************************
	    '  ADDED For THIS DEMO
	    ' **********************************************
	
	    '  Clear flags And steering/drive variables.
	    RecordingTrack = False
	    PlayingTrack = False
	    UserSteer = 0
	    UserDrive = 0
	
	    '  Before we can Print any text we must create a font
	    FontWhite = iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	    ' **********************************************
	
	    '  Create And set camera
	    Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	    iCameraFovSet(Camera,35.0)
	    iCameraLocationSet(Camera,_D3DXVECTOR3(23.0,10.0,-100.0))
	
	    '  Set sunlight
	    iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	    '  Create And show 'loading...' sprite
	    LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	    iSpriteRender(LoadingMessage,False)
	
	    '  Create static background
	    SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	    '  Create terrain static body And mesh
	    Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	    TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	    '  Create dynamic car-body body And mesh
	    CarBody=iBodySGCreate("simplecar_res\\carbody_.spg",0.13)
	    CarBodyMesh=iBodyMeshCreate("simplecar_res\\carbody.x",CarBody)
	
	    '  Set initial car position
	    iBodyLocationSet(CarBody,_D3DXVECTOR3(43.0,5.0,-100.0),True)
	
	    '  A little bit of damping is always good To prevent excessive speed And spin
	    iBodyDampingSet(CarBody,0.0,0.999,0.0,0.995)
	
	    '  Create car-body shadow mesh
	    CarBodyMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carbody_shadow.x",CarBody,500.0)
	
	    '  Enable car-terrain collision checking (create collision couple)
	    CarBodyTerrain=iBodyBodyCreate(CarBody,Terrain)
	    iBodyBodyFrictionSet(CarBodyTerrain,2.5)
	
	    ' ********************************************************
	    '  SIMULATION PARAMETERS: you can adjust car-dynamics here!
	
	    '  The friction produced by wheels, when they move sideways respect To terrain
	    SideFriction = 7.0
	    '  The friction produced by wheels, when they move parallel To rolling direction
	    RollingWiseFriction = 3.0
	    '  Suspension strength
	    SuspensionSpring = 10.0
	    '  Suspension hardness
	    SuspensionDamping = 1.0
	    '  Maximum wheel spin (degrees per second)
	    WheelSpin = 1600.0
	    '  Maximum wheel power applied To reach And keep maximum speed
	    WheelPower = 1.0
	
	    ' ********************************************************
	
	    '  Initialize steering And throttle
	    SteerAngle = 0.0
	    Spin = 0.0
	
	    '  Create FRONT-Left wheel dynamic body, mesh And shadow mesh
	    CarWheelFL=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	    CarWheelFLMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelFL)
	    CarWheelFLMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelFL,500.0)
	
	    '  Enable terrain-wheel collision checking
	    CarWheelFLTerrain=iBodyBodyCreate(CarWheelFL,Terrain)
	
	    '  Enable split-friction mode
	    iBodyBodySplitFrictionSet(CarWheelFLTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	
	    '  Create wheel joint (attach wheel-body To car-body) And set suspension parameters
	    CarWheelFLJoint = iWheelCreate(CarBody,CarWheelFL,_D3DXVECTOR3(-1.12,0.16,1.22),Null)
	    iWheelSuspensionSet(CarWheelFLJoint,SuspensionSpring,SuspensionDamping)
	
	    '  Create FRONT-Right wheel dynamic body, mesh And shadow mesh
	    CarWheelFR=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	    CarWheelFRMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelFR)
	    CarWheelFRMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelFR,500.0)
	
	    '  Enable terrain-wheel collision checking
	    CarWheelFRTerrain=iBodyBodyCreate(CarWheelFR,Terrain)
	
	    '  Enable split-friction mode
	    iBodyBodySplitFrictionSet(CarWheelFRTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	
	    '  Create wheel joint (attach wheel-body To car-body) And set suspension parameters
	    CarWheelFRJoint = iWheelCreate(CarBody,CarWheelFR,_D3DXVECTOR3(1.12,0.16,1.22),Null)
	    iWheelSuspensionSet(CarWheelFRJoint,SuspensionSpring,SuspensionDamping)
	
	    '  Create REAR-Left wheel dynamic body, mesh And shadow mesh
	    CarWheelRL=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	    CarWheelRLMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelRL)
	    CarWheelRLMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelRL,500.0)
	
	    '  Enable terrain-wheel collision checking
	    CarWheelRLTerrain=iBodyBodyCreate(CarWheelRL,Terrain)
	
	    '  Enable split-friction mode
	    iBodyBodySplitFrictionSet(CarWheelRLTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	
	    '  Create wheel joint (attach wheel-body To car-body) And set suspension parameters
	    CarWheelRLJoint = iWheelCreate(CarBody,CarWheelRL,_D3DXVECTOR3(-1.12,0.16,-1.10),Null)
	    iWheelSuspensionSet(CarWheelRLJoint,SuspensionSpring,SuspensionDamping)
	
	    '  Create REAR-Right wheel dynamic body, mesh And shadow mesh
	    CarWheelRR=iBodySGCreate("simplecar_res\\carwheel_.spg",0.13)
	    CarWheelRRMesh=iBodyMeshCreate("simplecar_res\\carwheel.x",CarWheelRR)
	    CarWheelRRMeshShadow=iMeshBodyShadowCasterCreate("simplecar_res\\carwheel_shadow.x",CarWheelRR,500.0)
	
	    '  Enable terrain-wheel collision checking
	    CarWheelRRTerrain=iBodyBodyCreate(CarWheelRR,Terrain)
	
	    '  Enable split-friction mode
	    iBodyBodySplitFrictionSet(CarWheelRRTerrain,SideFriction,0.0,RollingWiseFriction,0.0)
	
	    '  Create wheel joint (attach wheel-body To car-body) And set suspension parameters
	    CarWheelRRJoint = iWheelCreate(CarBody,CarWheelRR,_D3DXVECTOR3(1.12,0.16,-1.10),Null)
	    iWheelSuspensionSet(CarWheelRRJoint,SuspensionSpring,SuspensionDamping)
	
	    '  Enable all bodies
	    iBodyEnable(Terrain)
	    iBodyEnable(CarBody)
	    iBodyEnable(CarWheelFL)
	    iBodyEnable(CarWheelFR)
	    iBodyEnable(CarWheelRL)
	    iBodyEnable(CarWheelRR)
	
	    '  Hide 'loading...' sprite
	    iSpriteHide(LoadingMessage)
	
	    '  Show static background And all meshes
	    iSkyBoxShow(SkyBox)
	    iMeshShow(TerrainMesh)
	    iMeshShow(CarBodyMesh)
	    iMeshShow(CarBodyMeshShadow)
	    iMeshShow(CarWheelFLMesh)
	    iMeshShow(CarWheelFLMeshShadow)
	    iMeshShow(CarWheelFRMesh)
	    iMeshShow(CarWheelFRMeshShadow)
	    iMeshShow(CarWheelRLMesh)
	    iMeshShow(CarWheelRLMeshShadow)
	    iMeshShow(CarWheelRRMesh)
	    iMeshShow(CarWheelRRMeshShadow)
	EndMethod
	
	Method _Run()
	
	    '  Update camera orientation so that it always takes the car
	    Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	    iBodyLocationCM(CarBody,cameratarget)
	    iCameraLookAt(Camera,cameratarget,0.1)
	
	    ' **********************************************
	    '  ADDED For THIS DEMO
	    ' **********************************************
	
	    If (RecordingTrack)
	
	        '  Display 'recording' message.
	        iPrint("REC",_D3DXVECTOR2(-15.0,10.0),_D3DXVECTOR2(1.0,1.0),0.1,FontWhite)
	
	        '  Update timer.
	        TrackTime :+ 0.001
	
	        '  Check If user has changed steering.
	        If (iKeyDown(DIK_LEFT))
	            If (UserSteer <> -1) iTrackWriteSteer(-1)
	        ElseIf (iKeyDown(DIK_RIGHT))
	            If (UserSteer <>  1) iTrackWriteSteer( 1)
	        Else
	            If (UserSteer <>  0) iTrackWriteSteer( 0)
	        EndIf
	
	        '  Check If user has changed forward/reverse motion.
	        If (iKeyDown(DIK_UP))
	            If (UserDrive <>  1) iTrackWriteDrive( 1)
	        ElseIf (iKeyDown(DIK_DOWN))
	            If (UserDrive <> -1) iTrackWriteDrive(-1)
	        Else
	            If (UserDrive <>  0) iTrackWriteDrive( 0)
	        EndIf
	
	        '  Check If user has braked/stopped braking.
	        If (iKeyDown(DIK_LCONTROL) Or iKeyDown(DIK_RCONTROL))
	            If (UserBrake <>  1) iTrackWriteBrake(1)
	        Else
	            If (UserBrake <>  0) iTrackWriteBrake(0)
	        EndIf
	
	    Else If (PlayingTrack)
	
	        '  Display 'playing' message.
	        iPrint("PLAY",_D3DXVECTOR2(-15.0,10.0),_D3DXVECTOR2(1.0,1.0),0.1,FontWhite)
	
	        '  Update timer.
	        TrackTime :+ 0.001
	
	        '  Check If current time has passed time value of Next move.
	        If (TrackTime >= NextTime)
	
	            '  Read Next time value from track file.
			  NextTime=ReadLine(TrackFile).ToFloat()
	            'fscanf(TrackFile,"%f",NextTime)
							
	            '  If file has ended stop playback.
	            If (Eof(TrackFile))
	                iTrackStop()
	                Return
	            EndIf
	
	            '  Read Next move from track file.
			  'Local temp:String=ReadLine(TrackFile)
	            'Local Action:String=Chr(temp[0])
	            'Local Value:Int=temp[0..temp.length-1].ToInt()
			  Local Action:String=Chr(ReadByte(TrackFile))
			  Local Value:Int=ReadLine(TrackFile).ToInt()
			
	            'fscanf(TrackFile,"%c%i",Action,Value)
	
	            '  Update User~ variables based on Action.
	 		  Select Action
				 Case "S"
					UserSteer = Value
				 Case "D"
					UserDrive = Value
				 Case "B"
					UserBrake = Value
			  EndSelect
	
	        EndIf
	
	        '  Replay steering as specified by UserSteer variable.
	        If (UserSteer = -1)
	            iFloatTendTo(SteerAngle,20.0,0.3)
	        ElseIf (UserSteer = 1)
	            iFloatTendTo(SteerAngle,-20.0,0.3)
	        Else
			 iFloatTendTo(SteerAngle,0.0,0.2)
		   EndIf
	
	        '  Replay forward/reverse as specified by UserDrive variable.
	        If (UserDrive)
	            iFloatTendTo(Spin,-WheelSpin,0.4)
	        ElseIf (UserDrive)
	            iFloatTendTo(Spin,WheelSpin,0.4)
	        Else
			 iFloatTendTo(Spin,0.0,0.1)
		   EndIf
	
	    EndIf
	
	    '  Check If user has pressed the R, P Or S key - using the 'iKeyClick' function
	    '  which returns True when a key has been pressed And released (as opposed To
	    '  iKeyDown which returns True If a key is being HELD down).
	    If (iKeyClick(DIK_R))
	        iTrackRecord()
	    ElseIf (iKeyClick(DIK_P))
	        iTrackPlay()
	    ElseIf (iKeyClick(DIK_S))
	        iTrackStop()
	    EndIf
	
	    ' **********************************************
	
	    ' **********************************************
	    '  ADDED For THIS DEMO
	    ' **********************************************
	    '  Only allow user To control car If Not playing back.
	    If (Not PlayingTrack)
	        ' If Left/Right arrow keys are pressed, progressively increase/decrease the SteerAngle
	        ' variable (within -20 / 20 degrees).
	        ' If neither Left nor Right keys are pressed, progressively adjust SteerAngle variable
	        ' toward zero, in order To perform steering auto-centering.
	        If (iKeyDown(DIK_LEFT))
	            iFloatTendTo(SteerAngle,20.0,0.3)
	        ElseIf (iKeyDown(DIK_RIGHT))
	            iFloatTendTo(SteerAngle,-20.0,0.3)
	        Else
			 iFloatTendTo(SteerAngle,0.0,0.2)
		   EndIf
	    EndIf
	
	    '  Apply current SteerAngle To front wheels (set wheel joints steering parameter)
	    iWheelSteeringAngleSet(CarWheelFLJoint,SteerAngle)
	    iWheelSteeringAngleSet(CarWheelFRJoint,SteerAngle)
	
	    ' **********************************************
	    '  ADDED For THIS DEMO
	    ' **********************************************
	    '  Only allow user To control car If Not playing back.
	    If (Not PlayingTrack)
	        '  If up/down arrow keys are pressed, progressively decrease/increase the Spin
	        '  variable (within -WheelSpin / WheelSpin).
	        '  If neither up nor down keys are pressed, progressively adjust Spin variable
	        '  toward zero, in order To perform throttle auto-returning
	        If (iKeyDown(DIK_UP))
	            iFloatTendTo(Spin,-WheelSpin,0.4)
	        ElseIf (iKeyDown(DIK_DOWN))
	            iFloatTendTo(Spin,WheelSpin,0.4)
	        Else
			 iFloatTendTo(Spin,0.0,0.1)
		   EndIf
	    EndIf
	
	    '  Wheel power is controlled by two factors: the maximum spin permitted And the torque
	    '  intensity used To reach And keep such speed.
	    '  In order To simulate a simple one-gear system, in which wheel spin And power are
	    '  proportional To throttle depression, we let the user directly tune the
	    '  maximum spin with up/down arrow keys (previous section above).
	    '  Then we compute the actual wheel power (torque) so that it is proportional To
	    '  such user-adjusted Spin
	    Local power:Float = iFloatInterpolate(iFloatAbs(Spin),0.0,WheelSpin,WheelPower*0.1,WheelPower,False)
	
	    ' **********************************************
	    '  ADDED For THIS DEMO
	    ' **********************************************
	    '  Only allow user To control car If Not playing back.
	    If (PlayingTrack)
	        '  If UserBrake indicates, perform braking here (after power has been calculated).
	        If (UserBrake = 1)
	            Spin = 0.0
	            power = 1.5
	        EndIf
	    Else
	        '  However, If a [Ctrl] key is pressed, force Spin And power variables back To zero And 1.5f
	        '  respectively, To simulate brakes. The higher the power value, the stronger the brakes
	        If ((iKeyDown(DIK_LCONTROL)) Or (iKeyDown(DIK_RCONTROL)))
	            Spin = 0.0
	            power = 1.5
	        EndIf
	    EndIf
	
	    '  Apply user-adjusted spin And computed power To all 4 wheels
	    iWheelSpinSet(CarWheelFLJoint,Spin)
	    iWheelSpinSet(CarWheelFRJoint,Spin)
	    iWheelSpinSet(CarWheelRLJoint,Spin)
	    iWheelSpinSet(CarWheelRRJoint,Spin)
	    iWheelPowerSet(CarWheelFLJoint,power)
	    iWheelPowerSet(CarWheelFRJoint,power)
	    iWheelPowerSet(CarWheelRLJoint,power)
	    iWheelPowerSet(CarWheelRRJoint,power)
	
	    '  Apply gravity acceleration To car-body And wheel-bodies
	    iBodyAccelerationApply(CarBody,_D3DXVECTOR3(0.0,-9.80665,0.0))
	    iBodyAccelerationApply(CarWheelFL,_D3DXVECTOR3(0.0,-9.80665,0.0))
	    iBodyAccelerationApply(CarWheelFR,_D3DXVECTOR3(0.0,-9.80665,0.0))
	    iBodyAccelerationApply(CarWheelRL,_D3DXVECTOR3(0.0,-9.80665,0.0))
	    iBodyAccelerationApply(CarWheelRR,_D3DXVECTOR3(0.0,-9.80665,0.0))
	EndMethod

	Method _Exit()
	    '  Automatically stop recording on exit.
	    iTrackStop()
	EndMethod
EndType
	
