' Skinmesh bone control demo.

' A simple skinmesh bone viewer.
' The code will show you how To access And dynamically control each single bone of a skinmesh.
' It also shows how To control a specific bone While playing other bones animation normally.

' REMARKS:

' Note that when skinmesh animation speed is Not zero, bone orientations are updated (that
' is, they are 'forced' to the orientation specified in the played animation) before the
' Run() callback is executed.
' 
' It means that you should call iSkinMeshBoneOrientationSet() at every DLL loop
' in order To actually control a bone orientation While playing the animation For
' the rest of the bones normally.

' NOTE: in our code comments we assume you understand the SkinMesh sample code.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Include "..\common\3impact_cameraman.bmx"

Global Camera:CAMERA
Global FontWhite:FONT
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX

Global SkinMesh:SKINMESH

Global MessageString:String

Const MAX_BONES:Int = 1000
Global SkinMeshBone:SKINMESHBONE[MAX_BONES]
Global BoneEulerX:Float
Global BoneEulerY:Float
Global BoneEulerZ:Float
Global BoneCount:Int
Global BoneIndex:Int
Global BoneName:String
Global AnimSpeed:Float

Function MouseLook(mouselookx:Float Var,mlxspd:Float,mouselooky:Float Var,mlyspd:Float,reset:Int)

   ' A tricky Function To dynamically alter the contents
   ' of two Float variables by using the mouse.

   ' Float* mouselookx = pointer To the Float variable To alter by moving the mouse up/down
   ' Float mlxspd = variation speed factor (up/down)
   ' Float* mouselooky = pointer To the Float variable To alter by moving the mouse Left/Right
   ' Float mlyspd = variation speed factor (Left/Right)
   ' BOOL reset = whether To reset (True) the mouselook system

   Global mouselookxold:Float = 0.0
   Global mouselookyold:Float = 0.0
   If (reset)
      mouselookxold = mouselookx / mlxspd
      mouselookyold = mouselooky / mlyspd
      iMouseReset(_D3DXVECTOR2(0.5,0.5))
   Else
      mouselookx :/ mlxspd
      mouselooky :/ mlyspd
      mouselookx = mouselookxold + (0.5-iMouseY())
      mouselooky = mouselookyold + (iMouseX()-0.5)
      If (iFloatAbs(mouselookxold - mouselookx) > 0.45)
         mouselookxold = mouselookx
         iMouseReset(_D3DXVECTOR2(iMouseX(),0.5))
      EndIf
      If (iFloatAbs(mouselookyold - mouselooky) > 0.45)
         mouselookyold = mouselooky
         iMouseReset(_D3DXVECTOR2(0.5,iMouseY()))
      EndIf
      mouselookx :* mlxspd
      mouselooky :* mlyspd
   EndIf
EndFunction

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Local i:Int
	
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,0.0,-3.0))
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	   MessageString=""
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   ' Create a skinmesh from an .x file containing bone-based animation information
	   SkinMesh=iSkinMeshCreate("default_res\\ragdoll\\androanim.x",Null)
	
	   ' Place the animated mesh
	   iSkinMeshEulerOrientationSet(SkinMesh,0.0,180.0,0.0,"xyz")
	   iSkinMeshLocationSet(SkinMesh,_D3DXVECTOR3(0.0,-1.0,0.0))
	   ' Reset animation time To zero
	   iSkinMeshAnimationTimeSet(SkinMesh,0.0)
	   ' Set animation's initial speed
	   AnimSpeed = .05
	   iSkinMeshAnimationSpeedSet(SkinMesh,AnimSpeed)
	
	   ' Retrieve total number of bones in the skinmesh
	   BoneCount = iSkinMeshBonesCount(SkinMesh)
	   If (BoneCount > MAX_BONES) BoneCount = MAX_BONES
	   ' For each bone, create a skinmeshbone Object And store it in a
	   ' SKINMESHBONE* variable. We have defined an array of SKINMESHBONE* variables
	   ' For that actually, as it is more handy than a list of single variables.
	   For i=0 To BoneCount-1
	      SkinMeshBone[i] = iSkinMeshBoneCreate(SkinMesh,i)
	   Next
	   ' Set the initial bone To edit
	   BoneIndex = 7
	
	   ' Hide loading message sprite, show skybox And skinmesh
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iSkinMeshShow(SkinMesh)
	EndMethod
	Method _Run()

	   ' Update X  Z bone orientation according To current mouse location
	   MouseLook(BoneEulerX,30.0,BoneEulerZ,50.0,False)
	
	   ' Alter Y bone orientation on arrow key presses
	   If (iKeyDown(DIK_LEFT)) BoneEulerY :+ 0.75
	   If (iKeyDown(DIK_RIGHT)) BoneEulerY :- 0.75
	
	   ' Find a quaternion representing the current bone XYZ Euler orientation...
	   Local q:D3DXQUATERNION=_D3DXQUATERNION()
	   iQuaternionFromEulerAngles(q,BoneEulerX,BoneEulerY,BoneEulerZ,"xyz")
	   ' ...And assign it To the current bone (the bone with index = BoneIndex)
	   iSkinMeshBoneOrientationSet(SkinMeshBone[BoneIndex],q,2)
	
	   ' If the user presses [Space], set the animation speed To zero If it is 200,
	   ' And set it To 200 If it is zero
	   If (iKeyClick(DIK_SPACE))
	      If (AnimSpeed)
			 AnimSpeed = 0.0
	      Else
			 AnimSpeed = .05
		 EndIf
	      iSkinMeshAnimationSpeedSet(SkinMesh,AnimSpeed)
	   EndIf
	
	   ' If the user presses [A], increase the current bone index (switch To the Next bone)
	   If (iKeyClick(DIK_A))
	      If (BoneIndex < BoneCount-1)
	         BoneIndex:+1
	         ' Retrieve the quaternion representing the New bone orientation
	         iSkinMeshBoneOrientation(SkinMeshBone[BoneIndex],q,2)
	         ' Convert it To Euler angles
	         iQuaternionToEulerAngles(q,BoneEulerX,BoneEulerY,BoneEulerZ)
	         ' Reset the mouselook system To the New bone orientation, To prevent
	         ' the New bone To be suddenly oriented To previous bone's angles
	         MouseLook(BoneEulerX,30.0,BoneEulerZ,50.0,True)
	      EndIf
	   EndIf
	   ' If the user presses [Z], decrease the current bone index (switch To the previous bone)
	   If (iKeyClick(DIK_Z))
	      If (BoneIndex > 0)
	         BoneIndex:-1
	         ' Retrieve the quaternion representing the New bone orientation
	         iSkinMeshBoneOrientation(SkinMeshBone[BoneIndex],q,2)
	         ' Convert it To Euler angles
	         iQuaternionToEulerAngles(q,BoneEulerX,BoneEulerY,BoneEulerZ)
	         ' Reset the mouselook system To the New bone orientation, To prevent
	         ' the New bone To be suddenly oriented To previous bone's angles
	         MouseLook(BoneEulerX,30.0,BoneEulerZ,50.0,True)
	      EndIf
	   EndIf
	
	   ' Print top screen stats messages
	   MessageString="Use Mouse  arrow keys to rotate bone,"
	   iPrintCentered(MessageString,_D3DXVECTOR2(0.0,11.0),_D3DXVECTOR2(1.0,0.5),0.1,FontWhite,0.0)
	   MessageString="[A]/[Z] to change bone,"
	   iPrintCentered(MessageString,_D3DXVECTOR2(0.0,10.0),_D3DXVECTOR2(1.0,0.5),0.1,FontWhite,0.0)
	   MessageString="[Space] to stop/start animation."
	   iPrintCentered(MessageString,_D3DXVECTOR2(0.0,9.0),_D3DXVECTOR2(1.0,0.5),0.1,FontWhite,0.0)
	
	   ' Print bottom screen stats messages
	   iSkinMeshBoneName(SkinMeshBone[BoneIndex],BoneName)
	   MessageString=BoneName+" ("+BoneIndex+") ("+BoneCount+")"
	   iPrintCentered(MessageString,_D3DXVECTOR2(0.0,-10.0),_D3DXVECTOR2(1.0,0.5),0.1,FontWhite,0.0)
	   MessageString="RX: "+String.FromFloat(BoneEulerX)[0..4]+"RY: "+String.FromFloat(BoneEulerY)[0..4]+" RZ: "+String.FromFloat(BoneEulerZ)[0..4]
	   iPrintCentered(MessageString,_D3DXVECTOR2(0.0,-11.0),_D3DXVECTOR2(1.0,0.5),0.1,FontWhite,0.0)
	EndMethod
EndType
