' Simple mono And 3d sound demo

' NOTE: in our code comments we assume you understand the BallGame sample code.

' THIS SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. USE AT YOUR OWN RISK.

Strict
Framework _3Impact.Lib

Global camera:CAMERA
Global LoadingMessage:SPRITE
Global SkyBox:SKYBOX
Global Terrain:BODY
Global TerrainMesh:MESH
Global Ball:BODY
Global BallMesh:MESH
Global BallMeshShadow:MESH
Global BallTerrain:BODYBODY
Global FontWhite:FONT

' Defines Object variables For the sounds
Global BallMusic:SOUND
Global KeySound:SOUND

If Not i3ImpactOpen() RuntimeError "Unable to initialize the 3Impact game engine"
If Not i3ImpactSettings(_ISETTINGS(640,480,32,False,True,False)) RuntimeError "Settings failed"

Local engine:sample_3IEngine=New sample_3IEngine

engine.Start()

Type sample_3IEngine Extends _3IEngine

	Method _Init()
	   Camera=iCameraCreate(0.0,0.0,1.0,1.0)
	   iCameraLocationSet(Camera,_D3DXVECTOR3(0.0,3.0,-10.0))
	
	   iLightDirectionalSet(_D3DXVECTOR3(1.0,-1.0,1.0),_D3DXVECTOR4(1.0,1.0,1.0,0.35))
	
	   LoadingMessage=iSpriteCreate("default_res\\sprites\\loading.x",Null)
	   iSpriteRender(LoadingMessage,False)
	
	   FontWhite=iFontCreate("default_res\\fontwhite\\font.wid",Null)
	
	   SkyBox=iSkyBoxCreate("default_res\\skyboxes\\skybox01.sky")
	
	   Terrain=iBodyCreate("default_res\\terrain_5.00.ply")
	   TerrainMesh=iBodyMeshCreate("default_res\\terrain.x",Terrain)
	
	   Ball=iBodySGCreate("default_res\\ball_.spg",0.13)
	   BallMesh=iBodyMeshCreate("default_res\\ball.x",Ball)
	   iBodyLocationSet(Ball,_D3DXVECTOR3(0.0,5.0,0.0),True)
	   iBodyDampingSet(Ball,0.0,0.999,0.0,0.995)
	   BallMeshShadow=iMeshBodyShadowCasterCreate("default_res\\ball_shadow.x",Ball,500.0)
	
	   BallTerrain=iBodyBodyCreate(Ball,Terrain)
	   iBodyBodyFrictionSet(BallTerrain,10.0)
	
	   ' Loads a Short sound from a file in VorbisOgg compressed format,
	   ' To be played as a mono sound whenever [Enter] is pressed.
	   KeySound=iSoundOggCreate("default_res\\sounds\\key.ogg",False,0)
	   ' Loads a looping-music sound from a file in VorbisOgg compressed format,
	   ' To be played as a continuous 3d sound originating from the ball.
	   ' REMEMBER: sounds that are To be played in 3d must be mono in the source file!
	   BallMusic=iSoundOggCreate("default_res\\sounds\\monoloop.ogg",True,0)
	   ' Resets the sound source To the same location as the ball
	   iSoundLocationSet(BallMusic,_D3DXVECTOR3(0.0,5.0,0.0))
	   ' Sets the music volume To 1.0f. It means that the music volume will be
	   ' maximum when the ball is 1.0f meter away from the camera Or closer.
	   ' It will decrease progressively While the ball goes farther.
	   iSoundVolumeSet(BallMusic,1.0)
	   ' Starts playing the music as a loop.
	   iSoundPlay(BallMusic,True)
	
	   iBodyEnable(Terrain)
	   iBodyEnable(Ball)
	
	   iSpriteHide(LoadingMessage)
	   iSkyBoxShow(SkyBox)
	   iMeshShow(TerrainMesh)
	   iMeshShow(BallMesh)
	   iMeshShow(BallMeshShadow)
	
	EndMethod
	Method _Run()
	   Local cameratarget:D3DXVECTOR3=_D3DXVECTOR3()
	   iBodyLocationCM(Ball,cameratarget)
	   If (iKeyDown(DIK_SPACE)) iCameraLookAt(Camera,cameratarget,0.1)
	
	   Local acceleration:D3DXVECTOR3
	   Local cameraorientation:D3DXQUATERNION=_D3DXQUATERNION()
	
	   acceleration=_D3DXVECTOR3(0.0,0.0,0.0)
	   If (iKeyDown(DIK_LEFT)) acceleration.PlusEq(_D3DXVECTOR3(-10.0,0.0,0.0))
	   If (iKeyDown(DIK_RIGHT)) acceleration.PlusEq(_D3DXVECTOR3(10.0,0.0,0.0))
	   If (iKeyDown(DIK_UP)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,10.0))
	   If (iKeyDown(DIK_DOWN)) acceleration.PlusEq(_D3DXVECTOR3(0.0,0.0,-10.0))
	
	   iCameraOrientation(Camera,cameraorientation)
	   iVectorRotate(acceleration,acceleration,cameraorientation)
	
	   acceleration.y = 0.0
	
	   iBodyAccelerationApply(Ball,acceleration)
	
	   If (Not iKeyDown(DIK_LCONTROL))
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,-9.80665,0.0))
	   Else
		 iBodyAccelerationApply(Ball,_D3DXVECTOR3(0.0,9.80665,0.0))
	   EndIf
	
	   iPrintCentered("[Space] to track the ball",_D3DXVECTOR2(0.0,-11.5),_D3DXVECTOR2(0.5,0.5),0.1,FontWhite,0.0)
	
	   If (iKeyClick(DIK_RETURN))
	      ' This {...} section is executed once If the [Enter] key is pressed.
	      ' It is executed again only If the key is released And pressed again.
	
	      ' The Next line plays the key sound once (no loop)
	      iSoundPlay(KeySound,False)
	   EndIf
	
	   ' Keeps the source of the music sound at the same location as the ball
	   iSoundLocationSet(BallMusic,cameratarget)
	EndMethod
EndType
