' -------------------------------------------------------
' CAMERA
' -------------------------------------------------------

Type CAMERA Extends i3ImpactBase	
	
	Method Set(top:Float,left_:Float,width:Float,height:Float)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraSet(objptr,top,left_,width,height)
	EndMethod
	
	Method AutoWipeSet(wipeMode:Int)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraAutoWipeSet(objptr,wipeMode)
	EndMethod

	Method SpritesOnlySet(bEnable:Int)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraSpritesOnlySet(objptr,bEnable)
	EndMethod

	Method MirrorEnable(bRenderMirrored:Int=False)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraMirrorEnable(objptr,bRenderMirrored)
	EndMethod

	Method PreRenderCallbackEnable(bEnable:Int)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraPreRenderCallbackEnable(objptr,bEnable)
	EndMethod

	Method FovSet(FOVdeg:Float=60.0)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraFovSet(objptr,FOVdeg)
	EndMethod

	Method Fov:Float()
		If Not IsValid() RuntimeError "Invalid Camera"
		Return _iCameraFov(objptr)
	EndMethod

	Method ClipDistanceSet(distance:Float=10000.0)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraClipDistanceSet(objptr,distance)
	EndMethod
	
	Method EulerOrientationSet(firstRotation:Float,secondRotation:Float,thirdRotation:Float,rotationOrder:String)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraEulerOrientationSet(objptr,firstRotation,secondRotation,thirdRotation,rotationOrder)
	EndMethod

	Method OrientationSet(orientation:D3DXQUATERNION)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraOrientationSet(objptr,orientation.param())
	EndMethod

	Method Orientation(orientation:D3DXQUATERNION)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraOrientation(objptr,orientation.param())
	EndMethod

	Method Location(location:D3DXVECTOR3)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraLocation(objptr,location.param())
	EndMethod

	Method LocationSet(location:D3DXVECTOR3)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraLocationSet(objptr,location.param())
	EndMethod

	Method VelocitySet(location:D3DXVECTOR3)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraVelocitySet(objptr,location.param())
	EndMethod

	Method LookAt(location:D3DXVECTOR3,quickness:Float)
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraLookAt(objptr,location.param(),quickness)
	EndMethod

	Method Destroy()
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraDestroy(objptr)
		objptr=Null
	EndMethod

	Method Hide()
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraHide(objptr)
	EndMethod

	Method Show()
		If Not IsValid() RuntimeError "Invalid Camera"
		_iCameraShow(objptr)
	EndMethod															

	Method EffectSet(camera_effect:CAMERAEFFECT = Null)
		_iCameraEffectSet(objptr, _HANDLE(camera_effect))
	EndMethod
	
	Method EffectValueSet(data_name:String, data_buffer:Byte Ptr, buffer_length:Int)
		_iCameraEffectValueSet(objptr, data_name, data_buffer, buffer_length)
	EndMethod
	
	Method EffectTextureSet(texture_name:String, tex:TEXTURE)
		_iCameraEffectTextureSet(objptr, texture_name ,tex.objptr)
	EndMethod
	
	Method EffectStack(camera_effect:CAMERAEFFECT, scaling_factor:Float)
		_iCameraEffectStack(objptr, _HANDLE(camera_effect), scaling_factor)
	EndMethod
	
EndType

Function iCameraCreate:CAMERA(top:Float,left_:Float,width:Float,height:Float,generateFunc:Byte Ptr=Null)
	Return _g_3IEngine.CameraCreate(top,left_,width,height,generateFunc)
EndFunction 

Function iCameraSet(cam:CAMERA,top:Float,left_:Float,width:Float,height:Float)
	If Not cam RuntimeError "Camera is Null"
	cam.Set(top,left_,width,height)
EndFunction

Function iCameraAutoWipeSet(cam:CAMERA,wipeMode:Int)
	If Not cam RuntimeError "Camera is Null"
	cam.AutoWipeSet(wipeMode)
EndFunction

Function iCameraSpritesOnlySet(cam:CAMERA,bEnable:Int)
	If Not cam RuntimeError "Camera is Null"
	cam.SpritesOnlySet(bEnable)
EndFunction

Function iCameraMirrorEnable(cam:CAMERA,bRenderMirrored:Int=False)
	If Not cam RuntimeError "Camera is Null"
	cam.MirrorEnable(bRenderMirrored)
EndFunction

Function iCameraPreRenderCallbackEnable(cam:CAMERA,bEnable:Int)
	If Not cam RuntimeError "Camera is Null"
	cam.PreRenderCallbackEnable(bEnable)
EndFunction

Function iCameraFovSet(cam:CAMERA,FOVdeg:Float=60.0)
	If Not cam RuntimeError "Camera is Null"
	cam.FovSet(FOVdeg)
EndFunction

Function iCameraFov:Float(cam:CAMERA)
	If Not cam RuntimeError "Camera is Null"
	Return cam.Fov()
EndFunction

Function iCameraClipDistanceSet(cam:CAMERA,distance:Float=10000.0)
	If Not cam RuntimeError "Camera is Null"
	cam.ClipDistanceSet(distance)
EndFunction

Function iCameraEulerOrientationSet(cam:CAMERA,firstRotation:Float,secondRotation:Float,thirdRotation:Float,rotationOrder:String)
	If Not cam RuntimeError "Camera is Null"
	cam.EulerOrientationSet(firstRotation,secondRotation,thirdRotation,rotationOrder)
EndFunction

Function iCameraOrientationSet(cam:CAMERA,orientation:D3DXQUATERNION)
	If Not cam RuntimeError "Camera is Null"
	cam.OrientationSet(orientation)
EndFunction

Function iCameraOrientation(cam:CAMERA,orientation:D3DXQUATERNION)
	If Not cam RuntimeError "Camera is Null"
	cam.Orientation(orientation)
EndFunction

Function iCameraLocation(cam:CAMERA,location:D3DXVECTOR3)
	If Not cam RuntimeError "Camera is Null"
	cam.Location(location)
EndFunction

Function iCameraLocationSet(cam:CAMERA,location:D3DXVECTOR3)
	If Not cam RuntimeError "Camera is Null"
	cam.LocationSet(location)
EndFunction

Function iCameraVelocitySet(cam:CAMERA,location:D3DXVECTOR3)
	If Not cam RuntimeError "Camera is Null"
	cam.VelocitySet(location)
EndFunction

Function iCameraLookAt(cam:CAMERA,location:D3DXVECTOR3,quickness:Float)
	If Not cam RuntimeError "Camera is Null"
	cam.LookAt(location,quickness)
EndFunction

Function iCameraDestroy(cam:CAMERA)
	If Not cam RuntimeError "Camera is Null"
	cam.Destroy()
EndFunction

Function iCameraHide(cam:CAMERA)
	If Not cam RuntimeError "Camera is Null"
	cam.Hide()
EndFunction

Function iCameraShow(cam:CAMERA)
	If Not cam RuntimeError "Camera is Null"
	cam.Show()
EndFunction

Type CAMERAEFFECT Extends i3ImpactBase	

	Method Destroy()
		_iCameraEffectDestroy(objptr)
		objptr=Null
	EndMethod
EndType

Function iCameraEffectCreate:CAMERAEFFECT(shader_filename:String,generateFunc:Byte Ptr=Null)
	Return _g_3IEngine.CameraEffectCreate(shader_filename,generateFunc)
EndFunction

Function iCameraEffectDestroy(cam_effect:CAMERAEFFECT)
	cam_effect.Destroy()
EndFunction

Function iCameraEffectSet(camera:CAMERA, cam_effect:CAMERAEFFECT)
	camera.EffectSet(cam_effect)
EndFunction

Function iCameraEffectValueSet(camera:CAMERA,data_name:String,data_buffer:Byte Ptr,buffer_length:Int)
	camera.EffectValueSet(data_name, data_buffer, buffer_length)
EndFunction

Function iCameraEffectTextureSet(camera:CAMERA, texture_name:String, tex:TEXTURE)
	camera.EffectTextureSet(texture_name, tex)
EndFunction

Function iCameraEffectStack(camera:CAMERA, cam_effect:CAMERAEFFECT, scaling_factor:Float)
	camera.EffectStack(cam_effect, scaling_factor)
EndFunction

Extern	
	' CAMERA
	Function _iCameraCreate:Byte Ptr(top:Float,left_:Float,width:Float,height:Float)="iCameraCreate" ' returns CAMERA*
	Function _iCameraSet(CAMERA:Byte Ptr,top:Float,left_:Float,width:Float,height:Float)="iCameraSet"
	Function _iCameraAutoWipeSet(CAMERA:Byte Ptr,wipeMode:Int)="iCameraAutoWipeSet"
	Function _iCameraSpritesOnlySet(CAMERA:Byte Ptr,bEnable:Int)="iCameraSpritesOnlySet"
	Function _iCameraPreRenderCallbackEnable(CAMERA:Byte Ptr,bEnable:Int)="iCameraPreRenderCallbackEnable"
	Function _iCameraFovSet(CAMERA:Byte Ptr,FOVdeg:Float=60.0)="iCameraFovSet"
	Function _iCameraFov:Float(CAMERA:Byte Ptr)="iCameraFov"
	Function _iCameraClipDistanceSet(CAMERA:Byte Ptr,distance:Float=10000.0)="iCameraClipDistanceSet"
	Function _iCameraEulerOrientationSet(CAMERA:Byte Ptr,firstRotation:Float,secondRotation:Float,thirdRotation:Float,rotationOrder$z)="iCameraEulerOrientationSet"
	Function _iCameraOrientationSet(CAMERA:Byte Ptr,D3DXQUATERNION_orientation:Byte Ptr)="iCameraOrientationSet"
	Function _iCameraOrientation(CAMERA:Byte Ptr,D3DXQUATERNION_orientation:Byte Ptr)="iCameraOrientation"
	Function _iCameraLocation(CAMERA:Byte Ptr,D3DXVECTOR3_location:Byte Ptr)="iCameraLocation"
	Function _iCameraLocationSet(CAMERA:Byte Ptr,D3DXVECTOR3_location:Byte Ptr)="iCameraLocationSet"
	Function _iCameraVelocitySet(CAMERA:Byte Ptr,D3DXVECTOR3_location:Byte Ptr)="iCameraVelocitySet"
	Function _iCameraLookAt(CAMERA:Byte Ptr,D3DXVECTOR3_location:Byte Ptr,quickness:Float)="iCameraLookAt"
	Function _iCameraDestroy(CAMERA:Byte Ptr)="iCameraDestroy"
	Function _iCameraHide(CAMERA:Byte Ptr)="iCameraHide"
	Function _iCameraShow(CAMERA:Byte Ptr)="iCameraShow"
	Function _iCameraMirrorEnable(CAMERA:Byte Ptr,bRenderMirrored:Int=False)="iCameraMirrorEnable"

	Function _iCameraEffectCreate:Byte Ptr(shader_filename$z) = "iCameraEffectCreate"
	Function _iCameraEffectDestroy(CAMERAEFFECT:Byte Ptr) = "iCameraEffectDestroy"
	Function _iCameraEffectSet(CAMERA:Byte Ptr,CAMERAEFFECT:Byte Ptr) = "iCameraEffectSet"
	Function _iCameraEffectValueSet(CAMERA:Byte Ptr,data_name$z,data_buffer:Byte Ptr,buffer_length:Int) = "iCameraEffectValueSet"
	Function _iCameraEffectTextureSet(CAMERA:Byte Ptr,texture_name$z,TEXTURE:Byte Ptr) = "iCameraEffectTextureSet"
	Function _iCameraEffectStack(CAMERA:Byte Ptr,CAMERAEFFFECT:Byte Ptr, scaling_factor:Float) = "iCameraEffectStack"

EndExtern

